@@ -2,7 +2,7 @@
  * atari_curses.c - Curses based port code
  *
  * Copyright (c) 1995-1998 David Firth
- * Copyright (c) 1998-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (c) 1998-2005 Atari800 development team (see DOC/CREDITS)
 
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -22,22 +22,21 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "config.h"
+#include <string.h>
 #ifdef USE_NCURSES
-#include	<ncurses.h>
+#include <ncurses.h>
 #else
-#include	<curses.h>
+#include <curses.h>
 #endif
 
+#include "antic.h" /* ypos */
 #include "atari.h"
-#include "ataripcx.h"
-#include "config.h"
-#include "cpu.h"
+#include "gtia.h" /* COLPFx */
 #include "input.h"
-#include "monitor.h"
-#include "memory.h"
-#include "rt-config.h"	/* refresh_rate */
-#include "ui.h"
 #include "log.h"
+#include "monitor.h"
+#include "ui.h" /* alt_function */
 
 #ifdef SOUND
 #include "sound.h"
@@ -49,9 +48,9 @@
 #define CURSES_WIDE_1 3
 #define CURSES_WIDE_2 4
 
-static int curses_mode;
+static int curses_mode = CURSES_LEFT;
 
-UBYTE curses_screen[24][40];
+static int curses_screen[24][40];
 
 void Atari_Initialise(int *argc, char *argv[])
 {
@@ -92,206 +91,292 @@ void Atari_Initialise(int *argc, char *argv[])
 	nodelay(stdscr, 1);			/* Don't block for keypress */
 
 #ifdef SOUND
-   Sound_Initialise(argc, argv);
+	Sound_Initialise(argc, argv);
 #endif
 }
 
 int Atari_Exit(int run_monitor)
 {
-	int restart;
-
 	curs_set(1);
 	endwin();
+	Aflushlog();
 
-
-	if (run_monitor)
-		restart = monitor();
-	else
-		restart = FALSE;
-
-	if (restart) {
+	if (run_monitor && monitor()) {
 		curs_set(0);
+		return TRUE;
 	}
 #ifdef SOUND
-        else
-           Sound_Exit();
+	Sound_Exit();
 #endif
-	Aflushlog();
-	return restart;
+	return FALSE;
 }
 
-void Atari_DisplayScreen(UBYTE * screen)
+void curses_clear_screen(void)
 {
-	UWORD screenaddr;
+	int x;
+	int y;
+	for (y = 0; y < 24; y++)
+		for (x = 0; x < 40; x++)
+			curses_screen[y][x] = ' ';
+}
 
-	int xpos;
-	int ypos;
+void curses_clear_rectangle(int x1, int y1, int x2, int y2)
+{
+	int x;
+	int y;
+	for (y = y1; y <= y2; y++)
+		for (x = x1; x <= x2; x++)
+			curses_screen[y][x] = ' ';
+}
 
-	screenaddr = dGetWordAligned(88);
+void curses_putch(int x, int y, int ascii, UBYTE fg, UBYTE bg)
+{
+	/* handle line drawing chars */
+	switch (ascii) {
+	case 18:
+		ascii = '-';
+		break;
+	case 17:
+	case 3:
+		ascii = '/';
+		break;
+	case 26:
+	case 5:
+		ascii = '\\';
+		break;
+	case 124:
+		ascii = '|';
+		break;
+	default:
+		break;
+	}
+	if ((bg & 0xf) > (fg & 0xf))
+		curses_screen[y][x] = ascii + A_REVERSE;
+	else
+		curses_screen[y][x] = ascii;
+}
 
-	for (ypos = 0; ypos < 24; ypos++) {
-		for (xpos = 0; xpos < 40; xpos++) {
-			int ch;
+void curses_display_line(int anticmode, const UBYTE *screendata)
+{
+	int y;
+	int *p;
+	int w;
+	if (ypos < 32 || ypos >= 224)
+		return;
+	y = (ypos >> 3) - 4;
+	p = &(curses_screen[y][0]);
+	switch (anticmode) {
+	case 2:
+	case 3:
+	case 4:
+	case 5:
+		switch (DMACTL & 3) {
+		case 1:
+			p += 4;
+			w = 32;
+			break;
+		case 2:
+			w = 40;
+			break;
+		case 3:
+			screendata += 4;
+			w = 40;
+			break;
+		default:
+			return;
+		}
+		do {
+			static const int offset[8] = {
+				0x20,                       /* 0x00-0x1f: Numbers + !"$% etc. */
+				0x20,                       /* 0x20-0x3f: Upper Case Characters */
+				A_BOLD,                     /* 0x40-0x5f: Control Characters */
+				0,                          /* 0x60-0x7f: Lower Case Characters */
+				-0x80 + 0x20 + A_REVERSE,   /* 0x80-0x9f: Numbers + !"$% etc. */
+				-0x80 + 0x20 + A_REVERSE,   /* 0xa0-0xbf: Upper Case Characters */
+				-0x80 + A_BOLD + A_REVERSE, /* 0xc0-0xdf: Control Characters */
+				-0x80 + A_REVERSE           /* 0xe0-0xff: Lower Case Characters */
+			};
+			UBYTE c = *screendata++;
+			/* PDCurses prints '\x7f' as "^?".
+			   This causes problems if this is the last character in line.
+			   Use bold '>' for Atari's Tab symbol (filled right-pointing triangle). */
+			if (c == 0x7f)
+				*p = '>' + A_BOLD;
+			else if (c == 0xff)
+				*p = '>' + A_BOLD + A_REVERSE;
+			else
+				*p = c + offset[c >> 5];
+			p++;
+		} while (--w);
+		break;
+	case 6:
+	case 7:
+		switch (DMACTL & 3) {
+		case 1:
+			p += 12;
+			w = 16;
+			break;
+		case 2:
+			p += 10;
+			w = 20;
+			break;
+		case 3:
+			p += 8;
+			w = 24;
+			break;
+		default:
+			return;
+		}
+		{
+#define LIGHT_THRESHOLD 0x0c
+			int light[4];
+			light[0] = (COLPF0 & 0x0e) >= LIGHT_THRESHOLD ? 0x20 + A_BOLD : 0x20;
+			light[1] = (COLPF1 & 0x0e) >= LIGHT_THRESHOLD ? -0x40 + 0x20 + A_BOLD : -0x40 + 0x20;
+			light[2] = (COLPF2 & 0x0e) >= LIGHT_THRESHOLD ? -0x80 + 0x20 + A_BOLD : -0x80 + 0x20;
+			light[3] = (COLPF3 & 0x0e) >= LIGHT_THRESHOLD ? -0xc0 + 0x20 + A_BOLD : -0xc0 + 0x20;
+			do {
+				*p++ = *screendata + light[*screendata >> 6];
+				screendata++;
+			} while (--w);
+		}
+		break;
+	default:
+		break;
+	}
+}
 
-			ch = ui_is_active ? curses_screen[ypos][xpos] : dGetByte(screenaddr);
-
-			switch (ch & 0xe0) {
-			case 0x00:			/* Numbers + !"$% etc. */
-				ch = 0x20 + ch;
-				break;
-			case 0x20:			/* Upper Case Characters */
-				ch = 0x40 + (ch - 0x20);
-				break;
-			case 0x40:			/* Control Characters */
-				ch = ch + A_BOLD;
-				break;
-			case 0x60:			/* Lower Case Characters */
-				break;
-			case 0x80:			/* Number, !"$% etc. */
-				ch = 0x20 + (ch & 0x7f) + A_REVERSE;
-				break;
-			case 0xa0:			/* Upper Case Characters */
-				ch = 0x40 + ((ch & 0x7f) - 0x20) + A_REVERSE;
-				break;
-			case 0xc0:			/* Control Characters */
-				ch = (ch & 0x7f) + A_REVERSE + A_BOLD;
-				break;
-			case 0xe0:			/* Lower Case Characters */
-				ch = (ch & 0x7f) + A_REVERSE;
-				break;
-			}
-
-			switch (curses_mode & 0x0f) {
+void Atari_DisplayScreen(void)
+{
+	int x;
+	int y;
+	for (y = 0; y < 24; y++) {
+		for (x = 0; x < 40; x++) {
+			int ch = curses_screen[y][x];
+			switch (curses_mode) {
 			default:
 			case CURSES_LEFT:
-				move(ypos, xpos);
+				move(y, x);
 				break;
 			case CURSES_CENTRAL:
-				move(ypos, 20 + xpos);
+				move(y, 20 + x);
 				break;
 			case CURSES_RIGHT:
-				move(ypos, 40 + xpos);
+				move(y, 40 + x);
 				break;
 			case CURSES_WIDE_1:
-				move(ypos, xpos + xpos);
+				move(y, x + x);
 				break;
 			case CURSES_WIDE_2:
-				move(ypos, xpos + xpos);
+				move(y, x + x);
 				addch(ch);
-				ch = (ch & 0x80) | 0x20;
+				ch = ' ' + (ch & A_REVERSE);
 				break;
 			}
-
 			addch(ch);
-
-			screenaddr++;
 		}
 	}
-
 	refresh();
 }
 
 int Atari_Keyboard(void)
 {
-	int keycode;
-
-	keycode = getch();
+	int keycode = getch();
 
 #if 0
 	/* for debugging */
 	if (keycode > 0) {
 		Atari800_Exit(FALSE);
-		printf("%o (%x, %d)\n", keycode,keycode,keycode);
+		printf("keycode == %d (0x%x)\n", keycode, keycode);
 		exit(1);
 	}
 #endif
 
+	key_consol = CONSOL_NONE;
+
 	switch (keycode) {
 	case 0x01:
-		keycode = AKEY_CTRL_A;
+		keycode = AKEY_CTRL_a;
 		break;
 	case 0x02:
-		keycode = AKEY_CTRL_B;
+		keycode = AKEY_CTRL_b;
+		break;
+	case 0x03 :
+		keycode = AKEY_CTRL_c;
 		break;
-/*
-   case 0x03 :
-   keycode = AKEY_CTRL_C;
-   break;
- */
 	case 0x04:
-		keycode = AKEY_CTRL_D;
+		keycode = AKEY_CTRL_d;
 		break;
 	case 0x05:
-		keycode = AKEY_CTRL_E;
+		keycode = AKEY_CTRL_e;
 		break;
 	case 0x06:
-		keycode = AKEY_CTRL_F;
+		keycode = AKEY_CTRL_f;
 		break;
 	case 0x07:
-		keycode = AKEY_CTRL_G;
-		break;
-#ifndef SOLARIS2
-	case 0x08:
-		keycode = AKEY_CTRL_H;
-		break;
-#endif
-	case 0x09:
-		keycode = AKEY_CTRL_I;
+		keycode = AKEY_CTRL_g;
 		break;
 /*
-   case 0x0a :
-   keycode = AKEY_CTRL_J;
-   break;
+	case 0x08:
+		keycode = AKEY_CTRL_h;
+		break;
+ */
+	case 0x09:
+		keycode = AKEY_TAB;
+		break;
+/*
+	case 0x0a:
+		keycode = AKEY_CTRL_j;
+		break;
  */
 	case 0x0b:
-		keycode = AKEY_CTRL_K;
+		keycode = AKEY_CTRL_k;
 		break;
 	case 0x0c:
-		keycode = AKEY_CTRL_L;
+		keycode = AKEY_CTRL_l;
 		break;
 /*
-   case 0x0d :
-   keycode = AKEY_CTRL_M;
-   break;
+	case 0x0d:
+		keycode = AKEY_CTRL_m;
+		break;
  */
 	case 0x0e:
-		keycode = AKEY_CTRL_N;
+		keycode = AKEY_CTRL_n;
 		break;
 	case 0x0f:
-		keycode = AKEY_CTRL_O;
+		keycode = AKEY_CTRL_o;
 		break;
 	case 0x10:
-		keycode = AKEY_CTRL_P;
+		keycode = AKEY_CTRL_p;
 		break;
 	case 0x11:
-		keycode = AKEY_CTRL_Q;
+		keycode = AKEY_CTRL_q;
 		break;
 	case 0x12:
-		keycode = AKEY_CTRL_R;
+		keycode = AKEY_CTRL_r;
 		break;
 	case 0x13:
-		keycode = AKEY_CTRL_S;
+		keycode = AKEY_CTRL_s;
 		break;
 	case 0x14:
-		keycode = AKEY_CTRL_T;
+		keycode = AKEY_CTRL_t;
 		break;
 	case 0x15:
-		keycode = AKEY_CTRL_U;
+		keycode = AKEY_CTRL_u;
 		break;
 	case 0x16:
-		keycode = AKEY_CTRL_V;
+		keycode = AKEY_CTRL_v;
 		break;
 	case 0x17:
-		keycode = AKEY_CTRL_W;
+		keycode = AKEY_CTRL_w;
 		break;
 	case 0x18:
-		keycode = AKEY_CTRL_X;
+		keycode = AKEY_CTRL_x;
 		break;
 	case 0x19:
-		keycode = AKEY_CTRL_Y;
+		keycode = AKEY_CTRL_y;
 		break;
 	case 0x1a:
-		keycode = AKEY_CTRL_Z;
+		keycode = AKEY_CTRL_z;
 		break;
 	case '`':
 		keycode = AKEY_CAPSTOGGLE;
@@ -344,15 +429,27 @@ int Atari_Keyboard(void)
 	case '?':
 		keycode = AKEY_QUESTION;
 		break;
+#ifdef PADMINUS
+	case PADMINUS:
+#endif
 	case '-':
 		keycode = AKEY_MINUS;
 		break;
+#ifdef PADPLUS
+	case PADPLUS:
+#endif
 	case '+':
 		keycode = AKEY_PLUS;
 		break;
+#ifdef PADSTAR
+	case PADSTAR:
+#endif
 	case '*':
 		keycode = AKEY_ASTERISK;
 		break;
+#ifdef PADSLASH
+	case PADSLASH:
+#endif
 	case '/':
 		keycode = AKEY_SLASH;
 		break;
@@ -573,35 +670,50 @@ int Atari_Keyboard(void)
 		keycode = AKEY_ESCAPE;
 		break;
 	case KEY_F0 + 1:
-                keycode = AKEY_UI;
+		keycode = AKEY_UI;
 		break;
 	case KEY_F0 + 2:
-		key_consol &= 0x03;
+		key_consol &= ~CONSOL_OPTION;
 		keycode = AKEY_NONE;
 		break;
 	case KEY_F0 + 3:
-		key_consol &= 0x05;
+		key_consol &= ~CONSOL_SELECT;
 		keycode = AKEY_NONE;
 		break;
 	case KEY_F0 + 4:
-		key_consol &= 0x06;
+		key_consol &= ~CONSOL_START;
 		keycode = AKEY_NONE;
 		break;
 	case KEY_F0 + 5:
 		keycode = AKEY_WARMSTART;
 		break;
+#ifdef KEY_HELP
+	case KEY_HELP:
+#endif
+#ifdef KEY_SHELP
+	case KEY_SHELP:
+#endif
+#ifdef KEY_LHELP
+	case KEY_LHELP:
+#endif
 	case KEY_F0 + 6:
-		keycode = AKEY_PIL;
+		keycode = AKEY_HELP;
 		break;
+#ifdef KEY_BREAK
+	case KEY_BREAK:
+#endif
 	case KEY_F0 + 7:
 		keycode = AKEY_BREAK;
 		break;
 	case KEY_F0 + 8:
-		keycode = AKEY_COLDSTART;
+		keycode = Atari_Exit(TRUE) ? AKEY_NONE : AKEY_EXIT;
 		break;
 	case KEY_F0 + 9:
 		keycode = AKEY_EXIT;
 		break;
+	case KEY_F0 + 10:
+		keycode = AKEY_SCREENSHOT;
+		break;
 	case KEY_DOWN:
 		keycode = AKEY_DOWN;
 		break;
@@ -614,19 +726,101 @@ int Atari_Keyboard(void)
 	case KEY_UP:
 		keycode = AKEY_UP;
 		break;
-#ifdef SOLARIS2
 	case 8:
 	case 127:
-#else
-	case 127:
+#ifdef KEY_BACKSPACE
+# if KEY_BACKSPACE != 8 && KEY_BACKSPACE != 127
 	case KEY_BACKSPACE:
+# endif
 #endif
 		keycode = AKEY_BACKSPACE;
 		break;
+#ifdef PADENTER
+	case PADENTER:
+#endif
 	case KEY_ENTER:
 	case '\n':
 		keycode = AKEY_RETURN;
 		break;
+#ifdef KEY_HOME
+	case KEY_HOME:
+		keycode = AKEY_CLEAR;
+		break;
+#endif
+#ifdef KEY_CLEAR
+	case KEY_CLEAR:
+		keycode = AKEY_CLEAR;
+		break;
+#endif
+#ifdef KEY_IC
+	case KEY_IC:
+		keycode = AKEY_INSERT_CHAR;
+		break;
+#endif
+#ifdef KEY_IL
+	case KEY_IL:
+		keycode = AKEY_INSERT_LINE;
+		break;
+#endif
+#ifdef KEY_DC
+	case KEY_DC:
+		keycode = AKEY_DELETE_CHAR;
+		break;
+#endif
+#ifdef KEY_DL
+	case KEY_DL:
+		keycode = AKEY_DELETE_LINE;
+		break;
+#endif
+#ifdef KEY_STAB
+	case KEY_STAB:
+		keycode = AKEY_SETTAB;
+		break;
+#endif
+#ifdef KEY_CTAB
+	case KEY_CTAB:
+		keycode = AKEY_CLRTAB;
+		break;
+#endif
+#ifdef ALT_A
+	/* PDCurses specific */
+	case ALT_A:
+		alt_function = MENU_ABOUT;
+		keycode = AKEY_UI;
+		break;
+	case ALT_C:
+		alt_function = MENU_CARTRIDGE;
+		keycode = AKEY_UI;
+		break;
+	case ALT_D:
+		alt_function = MENU_DISK;
+		keycode = AKEY_UI;
+		break;
+	case ALT_L:
+		alt_function = MENU_LOADSTATE;
+		keycode = AKEY_UI;
+		break;
+	case ALT_O:
+		alt_function = MENU_SOUND;
+		keycode = AKEY_UI;
+		break;
+	case ALT_R:
+		alt_function = MENU_RUN;
+		keycode = AKEY_UI;
+		break;
+	case ALT_S:
+		alt_function = MENU_SAVESTATE;
+		keycode = AKEY_UI;
+		break;
+	case ALT_W:
+		alt_function = MENU_SOUND_RECORDING;
+		keycode = AKEY_UI;
+		break;
+	case ALT_Y:
+		alt_function = MENU_SYSTEM;
+		keycode = AKEY_UI;
+		break;
+#endif
 	default:
 		keycode = AKEY_NONE;
 		break;
@@ -646,98 +840,15 @@ int Atari_TRIG(int num)
 
 int main(int argc, char **argv)
 {
-	int test_val = 0;
-	int keycode = AKEY_NONE;
-
 	/* initialise Atari800 core */
 	if (!Atari800_Initialise(&argc, argv))
 		return 3;
 
 	/* main loop */
-	while (TRUE) {
-		keycode = Atari_Keyboard();
-
-		switch (keycode) {
-		case AKEY_COLDSTART:
-			Coldstart();
-			break;
-		case AKEY_WARMSTART:
-			Warmstart();
-			break;
-		case AKEY_EXIT:
-			Atari800_Exit(FALSE);
-			return 0;
-		case AKEY_UI:
-#ifdef SOUND
-			Sound_Pause();
-#endif
-			ui((UBYTE *)atari_screen);
-#ifdef SOUND
-			Sound_Continue();
-#endif
-			break;
-		case AKEY_SCREENSHOT:
-			Save_PCX_file(FALSE, Find_PCX_name());
-			break;
-		case AKEY_SCREENSHOT_INTERLACE:
-			Save_PCX_file(TRUE, Find_PCX_name());
-			break;
-		case AKEY_BREAK:
-			key_break = 1;
-			break;
-		default:
-			key_break = 0;
-			key_code = keycode;
-			break;
-		}
-
-		if (++test_val == refresh_rate) {
-			Atari800_Frame(EMULATE_FULL);
-#ifndef DONT_SYNC_WITH_HOST
-			atari_sync(); /* here seems to be the best place to sync */
-#endif
-			Atari_DisplayScreen((UBYTE *) atari_screen);
-			test_val = 0;
-		}
-		else {
-#ifdef VERY_SLOW
-			Atari800_Frame(EMULATE_BASIC);
-#else	/* VERY_SLOW */
-			Atari800_Frame(EMULATE_NO_SCREEN);
-#ifndef DONT_SYNC_WITH_HOST
-			atari_sync();
-#endif
-#endif	/* VERY_SLOW */
-		}
+	for (;;) {
+		key_code = Atari_Keyboard();
+		Atari800_Frame();
+		if (display_screen)
+			Atari_DisplayScreen();
 	}
 }
-/*
-$Log: atari_curses.c,v $
-Revision 1.11  2003/03/03 09:57:32  joy
-Ed improved autoconf again plus fixed some little things
-
-Revision 1.10  2003/02/24 09:32:40  joy
-header cleanup
-
-Revision 1.9  2003/02/08 23:52:17  joy
-little cleanup
-
-Revision 1.8  2002/08/07 08:59:06  joy
-last EOL is not necessary
-
-Revision 1.7  2002/08/07 08:40:37  joy
-Aflushlog added, -help added
-
-Revision 1.6  2001/12/12 15:24:37  joy
-F5 is a warm start on all platforms.
-
-Revision 1.5  2001/10/11 08:39:55  fox
-added curses_screen for UI
-
-Revision 1.4  2001/10/10 12:14:10  fox
-compilable now
-
-Revision 1.3  2001/04/08 05:54:48  knik
-sound_update call removed (moved to atari.c)
-
-*/
