@@ -1,45 +1,52 @@
-/* -------------------------------------------------------------------------- */
-
 /*
- * Atari Falcon/TT/Nova Backend for David Firth's Atari 800 emulator
+ * atari_falcon.c - Atari Falcon specific port code
  *
- * by Petr Stehlik & Karel Rous (C)1997-98  See file COPYRIGHT for policy.
+ * Copyright (c) 1997-1998 Petr Stehlik and Karel Rous
+ * Copyright (c) 1998-2005 Atari800 development team (see DOC/CREDITS)
  *
+ * This file is part of the Atari800 emulator project which emulates
+ * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
+ *
+ * Atari800 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Atari800 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Atari800; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* -------------------------------------------------------------------------- */
-
+#include "config.h"
 #include <osbind.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>		/* for free */
-#include <falcon.h>		/* for VsetRGB */
+#include <mint/falcon.h>	/* for VsetRGB */
 #include "falcon/xcb.h"		/* for NOVA screensaver */
 
-#include "config.h"
+#include "atari.h"
 #include "cpu.h"
 #include "colours.h"
 #include "ui.h"         /* for ui_is_active */
 #include "input.h"
-#include "ataripcx.h"
-#include "config.h"
+#include "screen.h"
 #include "antic.h"		/* for BITPL_SCR */
 #include "platform.h"
 #include "monitor.h"
 #include "sound.h"
 #include "log.h"
+#include "util.h"
 
-#include "gemfast.h"
-#include "aesbind.h"
-#include "vdibind.h"
+#include "gem.h"
 
 /* -------------------------------------------------------------------------- */
 
-#define FALSE 0
-#define TRUE 1
-
-extern int refresh_rate;
-
 int get_cookie(long cookie, long *value)
 {
 	long *cookiejar = (long *) Setexc(0x168, -1L);
@@ -116,7 +123,7 @@ static int HOST_WIDTH, HOST_HEIGHT, HOST_PLANES;
 #define CENTER_Y	((HOST_HEIGHT - EMUL_HEIGHT) / 2)
 #define CENTER		(CENTER_X + CENTER_Y * HOST_WIDTH)
 
-#if defined(SET_LED) && defined(NO_LED_ON_SCREEN)
+#ifdef SHOW_DISK_LED
 static int LED_timeout = 0;
 #endif
 
@@ -280,7 +287,7 @@ void Atari_Initialise(int *argc, char *argv[])
 
 	for (i = j = 1; i < *argc; i++) {
 		if (strcmp(argv[i], "-interlace") == 0) {
-			sscanf(argv[++i], "%d", &skip_N_frames);
+			skip_N_frames = Util_sscandec(argv[++i]);
 		}
 		else if (strcmp(argv[i], "-joyswap") == 0)
 			joyswap = TRUE;
@@ -292,10 +299,10 @@ void Atari_Initialise(int *argc, char *argv[])
 			delta_screen = TRUE;
 		else {
 			if (strcmp(argv[i], "-help") == 0) {
-				printf("\t-interlace x  Generate Falcon screen only every X frame\n");
-				printf("\t-joyswap      Exchange joysticks\n");
-				printf("\t-videl        direct VIDEL programming (Falcon/VGA only)\n");
-				printf("\t-delta        delta screen output (differences only)\n");
+				Aprint("\t-interlace x  Generate Falcon screen only every X frame\n");
+				Aprint("\t-joyswap      Exchange joysticks\n");
+				Aprint("\t-videl        direct VIDEL programming (Falcon/VGA only)\n");
+				Aprint("\t-delta        delta screen output (differences only)\n");
 			}
 
 			argv[j++] = argv[i];
@@ -338,6 +345,8 @@ void Atari_Initialise(int *argc, char *argv[])
 	/* check for NOVA graphics card */
 	if (get_cookie('NOVA', &NOVA_xcb))
 		bitplanes = FALSE;
+	else if (get_cookie('fVDI', NULL))
+		bitplanes = FALSE;
 
 	/* GEM init */
 	appl_init();
@@ -486,15 +495,16 @@ inline long DoubleSizeIt(short data)
 	return result;
 }
 
-void Atari_DisplayScreen(UBYTE *screen)
+void Atari_DisplayScreen(void)
 {
+	UBYTE *screen = (UBYTE *) atari_screen;
 	static int i = 0;
 
 /*
 	if (! draw_display)
 		return;
 */
-#if defined(SET_LED) && defined(NO_LED_ON_SCREEN)
+#ifdef SHOW_DISK_LED
 	if (LED_timeout)
 		if (--LED_timeout == 0)
 			Atari_Set_LED(0);
@@ -538,7 +548,7 @@ void Atari_DisplayScreen(UBYTE *screen)
 		UBYTE *ptr_dest = kam + CENTER;
 		int j;
 
-		for(j=0; j<ATARI_HEIGHT; j++) {
+		for (j = 0; j < ATARI_HEIGHT; j++) {
 			short cycles;
 			long *long_ptr_from = ptr_from;
 			long *long_ptr_mirror = ptr_mirror;
@@ -553,25 +563,25 @@ void Atari_DisplayScreen(UBYTE *screen)
 						if (data == *long_ptr_mirror++)
 							long_ptr_dest+=2;
 						else {
-							long data2 = DoubleSizeIt((short)data);
-							long data1 = DoubleSizeIt((short)(data >> 16));
-							*(long_ptr_dest + HOST_WIDTH/4) = data1;
+							long data2 = DoubleSizeIt((short) data);
+							long data1 = DoubleSizeIt((short) (data >> 16));
+							*(long_ptr_dest + HOST_WIDTH / 4) = data1;
 							*long_ptr_dest++ = data1;
-							*(long_ptr_dest + HOST_WIDTH/4) = data2;
+							*(long_ptr_dest + HOST_WIDTH / 4) = data2;
 							*long_ptr_dest++ = data2;
 						}
-					} while(cycles--);
+					} while (cycles--);
 				}
 				else {
 					do {
 						long data = *long_ptr_from++;
-						long data2 = DoubleSizeIt((short)data);
-						long data1 = DoubleSizeIt((short)(data >> 16));
-						*(long_ptr_dest + HOST_WIDTH/4) = data1;
+						long data2 = DoubleSizeIt((short) data);
+						long data1 = DoubleSizeIt((short) (data >> 16));
+						*(long_ptr_dest + HOST_WIDTH / 4) = data1;
 						*long_ptr_dest++ = data1;
-						*(long_ptr_dest + HOST_WIDTH/4) = data2;
+						*(long_ptr_dest + HOST_WIDTH / 4) = data2;
 						*long_ptr_dest++ = data2;
-					} while(cycles--);
+					} while (cycles--);
 				}
 				ptr_dest += HOST_WIDTH;
 			}
@@ -591,7 +601,7 @@ void Atari_DisplayScreen(UBYTE *screen)
 						CHECK_AND_WRITE
 						CHECK_AND_WRITE
 						CHECK_AND_WRITE
-					} while(cycles--);
+					} while (cycles--);
 				}
 				else {
 					do {
@@ -599,7 +609,7 @@ void Atari_DisplayScreen(UBYTE *screen)
 						*long_ptr_dest++ = *long_ptr_from++;
 						*long_ptr_dest++ = *long_ptr_from++;
 						*long_ptr_dest++ = *long_ptr_from++;
-					} while(cycles--);
+					} while (cycles--);
 				}
 			}
 
@@ -610,7 +620,7 @@ void Atari_DisplayScreen(UBYTE *screen)
 	}
 }
 
-#if defined(SET_LED) && defined(NO_LED_ON_SCREEN)
+#ifdef SHOW_DISK_LED
 void Atari_Set_LED(int how)
 {
 	if (how) {
@@ -1114,9 +1124,8 @@ int Atari_Keyboard(void)
 			case 0x3f:			/* F5 */
 				keycode = shift_key ? AKEY_COLDSTART : AKEY_WARMSTART;
 				break;
-			case 0x40:			/* F6 - used to be PILL mode switch */
-				/* keycode = AKEY_PIL; */
-				keycode = AKEY_SCREENSHOT;
+			case 0x40:			/* F6 */
+				keycode = AKEY_HELP;
 				break;
 			case 0x41:			/* F7 */
 				keycode = AKEY_BREAK;
@@ -1196,18 +1205,6 @@ int Atari_TRIG(int num)
 
 /* -------------------------------------------------------------------------- */
 
-int Atari_POT(int num)
-{
-	return 228;
-}
-
-/* -------------------------------------------------------------------------- */
-
-int Atari_PEN(int vertical)
-{
-	return vertical ? 0xff : 0;
-}
-
 int main(int argc, char **argv)
 {
 	/* initialise Atari800 core */
@@ -1215,61 +1212,10 @@ int main(int argc, char **argv)
 		return 3;
 
 	/* main loop */
-	while (TRUE) {
-		static int test_val = 0;
-		int keycode = Atari_Keyboard();
-
-		switch (keycode) {
-		case AKEY_COLDSTART:
-			Coldstart();
-			break;
-		case AKEY_WARMSTART:
-			Warmstart();
-			break;
-		case AKEY_EXIT:
-			Atari800_Exit(FALSE);
-			exit(1);
-		case AKEY_UI:
-#ifdef SOUND
-			Sound_Pause();
-#endif
-			ui((UBYTE *)atari_screen);
-#ifdef SOUND
-			Sound_Continue();
-#endif
-			break;
-		case AKEY_SCREENSHOT:
-			Save_PCX_file(FALSE, Find_PCX_name());
-			break;
-		case AKEY_SCREENSHOT_INTERLACE:
-			Save_PCX_file(TRUE, Find_PCX_name());
-			break;
-		case AKEY_BREAK:
-			key_break = 1;
-			break;
-		default:
-			key_break = 0;
-			key_code = keycode;
-			break;
-		}
-
-		if (++test_val == refresh_rate) {
-			Atari800_Frame(EMULATE_FULL);
-#ifndef DONT_SYNC_WITH_HOST
-			atari_sync(); /* here seems to be the best place to sync */
-#endif
-			Atari_DisplayScreen((UBYTE *) atari_screen);
-			test_val = 0;
-		}
-		else {
-#ifdef VERY_SLOW
-			Atari800_Frame(EMULATE_BASIC);
-#else	/* VERY_SLOW */
-			Atari800_Frame(EMULATE_NO_SCREEN);
-#ifndef DONT_SYNC_WITH_HOST
-			atari_sync();
-#endif
-#endif	/* VERY_SLOW */
-		}
+	for (;;) {
+		key_code = Atari_Keyboard();
+		Atari800_Frame();
+		if (display_screen)
+			Atari_DisplayScreen();
 	}
 }
