@@ -2,7 +2,7 @@
  * cartridge.c - cartridge emulation
  *
  * Copyright (C) 2001-2003 Piotr Fusik
- * Copyright (C) 2001-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2001-2005 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -22,17 +22,21 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
+#include <string.h>
 
 #include "atari.h"
+#include "binload.h" /* loading_basic */
 #include "cartridge.h"
 #include "memory.h"
 #include "pia.h"
-#include "rt-config.h"
 #include "rtime.h"
+#include "util.h"
+#ifndef BASIC
 #include "statesav.h"
+#endif
 
 int cart_kb[CART_LAST_SUPPORTED + 1] = {
 	0,
@@ -75,21 +79,23 @@ int cart_kb[CART_LAST_SUPPORTED + 1] = {
 	512,/* CART_SWXEGS_512 */
 	1024,/* CART_SWXEGS_1024 */
 	8,  /* CART_PHOENIX_8 */
-	16  /* CART_BLIZZARD_16 */
+	16, /* CART_BLIZZARD_16 */
+	128,/* CART_ATMAX_128 */
+	1024 /* CART_ATMAX_1024 */
 };
 
 int CART_IsFor5200(int type)
 {
 	switch (type) {
-		case CART_5200_32:
-		case CART_5200_EE_16:
-		case CART_5200_40:
-		case CART_5200_NS_16:
-		case CART_5200_8:
-		case CART_5200_4:
-			return TRUE;
-		default:
-			break;
+	case CART_5200_32:
+	case CART_5200_EE_16:
+	case CART_5200_40:
+	case CART_5200_NS_16:
+	case CART_5200_8:
+	case CART_5200_4:
+		return TRUE;
+	default:
+		break;
 	}
 	return FALSE;
 }
@@ -102,7 +108,7 @@ static int bank;
 
 /* DB_32, XEGS_32, XEGS_64, XEGS_128, XEGS_256, XEGS_512, XEGS_1024 */
 /* SWXEGS_32, SWXEGS_64, SWXEGS_128, SWXEGS_256, SWXEGS_512, SWXEGS_1024 */
-static void set_bank_809F(int b)
+static void set_bank_809F(int b, int main)
 {
 	if (b != bank) {
 		if (b & 0x80) {
@@ -113,6 +119,8 @@ static void set_bank_809F(int b)
 			Cart809F_Enable();
 			CartA0BF_Enable();
 			CopyROM(0x8000, 0x9fff, cart_image + b * 0x2000);
+			if (bank & 0x80)
+				CopyROM(0xa000, 0xbfff, cart_image + main);
 		}
 		bank = b;
 	}
@@ -142,7 +150,7 @@ static void set_bank_A0BF(int b)
 			CartA0BF_Disable();
 		else {
 			CartA0BF_Enable();
-			CopyROM(0xa000, 0xbfff, cart_image + (~b & 0x0007) * 0x2000);
+			CopyROM(0xa000, 0xbfff, cart_image + (~b & 7) * 0x2000);
 		}
 		bank = b;
 	}
@@ -155,7 +163,7 @@ static void set_bank_A0BF_WILL64(int b)
 			CartA0BF_Disable();
 		else {
 			CartA0BF_Enable();
-			CopyROM(0xa000, 0xbfff, cart_image + ( b & 7 ) * 0x2000);
+			CopyROM(0xa000, 0xbfff, cart_image + (b & 7) * 0x2000);
 		}
 		bank = b;
 	}
@@ -168,7 +176,35 @@ static void set_bank_A0BF_WILL32(int b)
 			CartA0BF_Disable();
 		else {
 			CartA0BF_Enable();
-			CopyROM(0xa000, 0xbfff, cart_image + ( b & 3 ) * 0x2000);
+			CopyROM(0xa000, 0xbfff, cart_image + (b & 3) * 0x2000);
+		}
+		bank = b;
+	}
+}
+
+static void set_bank_A0BF_ATMAX128(int b)
+{
+	if (b != bank) {
+		if (b >= 0x20)
+			return;
+		else if (b >= 0x10)
+			CartA0BF_Disable();
+		else {
+			CartA0BF_Enable();
+			CopyROM(0xa000, 0xbfff, cart_image + b * 0x2000);
+		}
+		bank = b;
+	}
+}
+
+static void set_bank_A0BF_ATMAX1024(int b)
+{
+	if (b != bank) {
+		if (b >= 0x80)
+			CartA0BF_Disable();
+		else {
+			CartA0BF_Enable();
+			CopyROM(0xa000, 0xbfff, cart_image + b * 0x2000);
 		}
 		bank = b;
 	}
@@ -214,11 +250,13 @@ static void CART_Access(UWORD addr)
 				break;
 			/* case 0x02:
 			case 0x06: */
+			default:
+				break;
 			}
 		set_bank_A0AF(b, 0x3000);
 		break;
 	case CART_DB_32:
-		set_bank_809F(addr & 0x03);
+		set_bank_809F(addr & 0x03, 0x6000);
 		break;
 	case CART_WILL_64:
 		set_bank_A0BF_WILL64(addr);
@@ -262,6 +300,12 @@ static void CART_Access(UWORD addr)
 		Cart809F_Disable();
 		CartA0BF_Disable();
 		break;
+	case CART_ATMAX_128:
+		set_bank_A0BF_ATMAX128(addr & 0xff);
+		break;
+	case CART_ATMAX_1024:
+		set_bank_A0BF_ATMAX1024(addr & 0xff);
+		break;
 	default:
 		break;
 	}
@@ -285,22 +329,22 @@ void CART_PutByte(UWORD addr, UBYTE byte)
 	}
 	switch (cart_type) {
 	case CART_XEGS_32:
-		set_bank_809F(byte & 0x03);
+		set_bank_809F(byte & 0x03, 0x6000);
 		break;
 	case CART_XEGS_64:
-		set_bank_809F(byte & 0x07);
+		set_bank_809F(byte & 0x07, 0xe000);
 		break;
 	case CART_XEGS_128:
-		set_bank_809F(byte & 0x0f);
+		set_bank_809F(byte & 0x0f, 0x1e000);
 		break;
 	case CART_XEGS_256:
-		set_bank_809F(byte & 0x1f);
+		set_bank_809F(byte & 0x1f, 0x3e000);
 		break;
 	case CART_XEGS_512:
-		set_bank_809F(byte & 0x3f);
+		set_bank_809F(byte & 0x3f, 0x7e000);
 		break;
 	case CART_XEGS_1024:
-		set_bank_809F(byte & 0x7f);
+		set_bank_809F(byte & 0x7f, 0xfe000);
 		break;
 	case CART_ATRAX_128:
 		if (byte & 0x80) {
@@ -340,22 +384,22 @@ void CART_PutByte(UWORD addr, UBYTE byte)
 		set_bank_80BF(byte & 0xbf);
 		break;
 	case CART_SWXEGS_32:
-		set_bank_809F(byte & 0x83);
+		set_bank_809F(byte & 0x83, 0x6000);
 		break;
 	case CART_SWXEGS_64:
-		set_bank_809F(byte & 0x87);
+		set_bank_809F(byte & 0x87, 0xe000);
 		break;
 	case CART_SWXEGS_128:
-		set_bank_809F(byte & 0x8f);
+		set_bank_809F(byte & 0x8f, 0x1e000);
 		break;
 	case CART_SWXEGS_256:
-		set_bank_809F(byte & 0x9f);
+		set_bank_809F(byte & 0x9f, 0x3e000);
 		break;
 	case CART_SWXEGS_512:
-		set_bank_809F(byte & 0xbf);
+		set_bank_809F(byte & 0xbf, 0x7e000);
 		break;
 	case CART_SWXEGS_1024:
-		set_bank_809F(byte);
+		set_bank_809F(byte, 0xfe000);
 		break;
 	default:
 		CART_Access(addr);
@@ -409,7 +453,7 @@ UBYTE BountyBob1_GetByte(UWORD addr)
 			return 0;
 		}
 	}
-	return memory[addr];
+	return dGetByte(addr);
 }
 
 UBYTE BountyBob2_GetByte(UWORD addr)
@@ -425,7 +469,7 @@ UBYTE BountyBob2_GetByte(UWORD addr)
 			return 0;
 		}
 	}
-	return memory[addr];
+	return dGetByte(addr);
 }
 
 void BountyBob1_PutByte(UWORD addr, UBYTE value)
@@ -455,7 +499,7 @@ void BountyBob2_PutByte(UWORD addr, UBYTE value)
 }
 #endif
 
-int CART_Checksum(UBYTE *image, int nbytes)
+int CART_Checksum(const UBYTE *image, int nbytes)
 {
 	int checksum = 0;
 	while (nbytes > 0) {
@@ -465,7 +509,7 @@ int CART_Checksum(UBYTE *image, int nbytes)
 	return checksum;
 }
 
-int CART_Insert(char *filename)
+int CART_Insert(const char *filename)
 {
 	FILE *fp;
 	int len;
@@ -480,21 +524,16 @@ int CART_Insert(char *filename)
 	if (fp == NULL)
 		return CART_CANT_OPEN;
 	/* check file length */
-	fseek(fp, 0L, SEEK_END);
-	len = ftell(fp);
-	fseek(fp, 0L, SEEK_SET);
-        
+	len = Util_flen(fp);
+	Util_rewind(fp);
+
 	/* Save Filename for state save */
 	strcpy(cart_filename, filename);
 
 	/* if full kilobytes, assume it is raw image */
 	if ((len & 0x3ff) == 0) {
 		/* alloc memory and read data */
-		cart_image = malloc(len);
-		if (cart_image == NULL) {
-			fclose(fp);
-			return CART_BAD_FORMAT;
-		}
+		cart_image = (UBYTE *) Util_malloc(len);
 		fread(cart_image, 1, len, fp);
 		fclose(fp);
 		/* find cart type */
@@ -529,11 +568,7 @@ int CART_Insert(char *filename)
 			int checksum;
 			len = cart_kb[type] << 10;
 			/* alloc memory and read data */
-			cart_image = malloc(len);
-			if (cart_image == NULL) {
-				fclose(fp);
-				return CART_BAD_FORMAT;
-			}
+			cart_image = (UBYTE *) Util_malloc(len);
 			fread(cart_image, 1, len, fp);
 			fclose(fp);
 			checksum = (header[8] << 24) |
@@ -561,15 +596,8 @@ void CART_Remove(void)
 
 void CART_Start(void) {
 	if (machine_type == MACHINE_5200) {
-#ifndef PAGED_ATTRIB
 		SetROM(0x4ff6, 0x4ff9);		/* disable Bounty Bob bank switching */
 		SetROM(0x5ff6, 0x5ff9);
-#else
-		readmap[0x4f] = NULL;
-		readmap[0x5f] = NULL;
-		writemap[0x4f] = ROM_PutByte;
-		writemap[0x5f] = ROM_PutByte;
-#endif
 		switch (cart_type) {
 		case CART_5200_32:
 			CopyROM(0x4000, 0xbfff, cart_image);
@@ -732,7 +760,7 @@ void CART_Start(void) {
 			if (machine_type == MACHINE_OSA || machine_type == MACHINE_OSB) {
 				Cart809F_Enable();
 				CopyROM(0x8000, 0x9fff, cart_image);
-				if (!disable_basic && have_basic) {
+				if ((!disable_basic || loading_basic) && have_basic) {
 					CartA0BF_Enable();
 					CopyROM(0xa000, 0xbfff, atari_basic);
 					break;
@@ -756,10 +784,22 @@ void CART_Start(void) {
 			CopyROM(0x8000, 0xbfff, cart_image);
 			bank = 0;
 			break;
+		case CART_ATMAX_128:
+			Cart809F_Disable();
+			CartA0BF_Enable();
+			CopyROM(0xa000, 0xbfff, cart_image);
+			bank = 0;
+			break;
+		case CART_ATMAX_1024:
+			Cart809F_Disable();
+			CartA0BF_Enable();
+			CopyROM(0xa000, 0xbfff, cart_image + 0xfe000);
+			bank = 0x7f;
+			break;
 		default:
 			Cart809F_Disable();
 			if ((machine_type == MACHINE_OSA || machine_type == MACHINE_OSB)
-			 && !disable_basic && have_basic) {
+			 && (!disable_basic || loading_basic) && have_basic) {
 				CartA0BF_Enable();
 				CopyROM(0xa000, 0xbfff, atari_basic);
 				break;
@@ -770,64 +810,36 @@ void CART_Start(void) {
 	}
 }
 
-void CARTStateRead( void )
+#ifndef BASIC
+
+void CARTStateRead(void)
 {
-    int savedCartType = CART_NONE;;
-    UWORD namelen;
-    FILE *fp;
-    char filename[FILENAME_MAX];
-    
-    /* Set the filename to NULL, in case the read fails */
-    filename[0] = 0;
-    /* Read the cart type from the file.  If there is no cart type, becaused we have
-       reached the end of the file, this will just default to CART_NONE */
-    ReadINT(&savedCartType, 1);
-    if (savedCartType != CART_NONE) {
-        /* Read the length of the saved cartridge filename */
-        ReadUWORD(&namelen, 1);
-        if (namelen != 0) {
-            if (namelen > (FILENAME_MAX-1))
-                namelen = FILENAME_MAX-1;
-            /* Read the filename, and add the terminating zero for C-string */
-            ReadUBYTE(filename, namelen);
-            filename[namelen] = 0;
-            /* Check to make sure the filename is valid */
-            fp = fopen(filename, "rb");
-            if (fp == NULL)
-		return;
-            else
-                fclose(fp);
-            /* Insert the cartridge...*/
-            if (CART_Insert(filename) >= 0) {
-                /* And set the type to the saved type, in case it was a raw cartridge image */
-                cart_type = savedCartType;
-                CART_Start();
-                }
-            }
-        }
+	int savedCartType = CART_NONE;
+
+	/* Read the cart type from the file.  If there is no cart type, becaused we have
+	   reached the end of the file, this will just default to CART_NONE */
+	ReadINT(&savedCartType, 1);
+	if (savedCartType != CART_NONE) {
+		char filename[FILENAME_MAX];
+		ReadFNAME(filename);
+		if (filename[0]) {
+			/* Insert the cartridge... */
+			if (CART_Insert(filename) >= 0) {
+				/* And set the type to the saved type, in case it was a raw cartridge image */
+				cart_type = savedCartType;
+				CART_Start();
+			}
+		}
+	}
 }
 
-void CARTStateSave( void)
+void CARTStateSave(void)
 {
-    UWORD namelen;
-    char *filename;
-    char dirname[FILENAME_MAX];
-    
-    /* Save the cartridge type, or CART_NONE if there isn't one...*/
-    SaveINT(&cart_type,1);
-    if (cart_type != CART_NONE) {
-        /* Check to see if file is in application tree, if so, just save as 
-           relative path....*/
-        getcwd(dirname, FILENAME_MAX);
-        if (strncmp(cart_filename, dirname, strlen(dirname)) == 0)
-            filename = &cart_filename[strlen(dirname)+1];
-        else
-            filename = cart_filename;
-
-        namelen = strlen(filename);
-        /* Save the length of the filename, followed by the filename */
-        SaveUWORD(&namelen, 1);
-        SaveUBYTE(filename, namelen);
-        }
+	/* Save the cartridge type, or CART_NONE if there isn't one...*/
+	SaveINT(&cart_type, 1);
+	if (cart_type != CART_NONE) {
+		SaveFNAME(cart_filename);
+	}
 }
 
+#endif
