@@ -9,8 +9,9 @@
 
 void CASSETTE_Initialise(int *argc, char *argv[]);
 
-int CASSETTE_CheckFile(char *filename, FILE **fp, char *description, int *last_block, int *isCAS);
-int CASSETTE_Insert(char *filename);
+int CASSETTE_CheckFile(const char *filename, FILE **fp, char *description, int *last_block, int *isCAS);
+int CASSETTE_CreateFile(const char *filename, FILE **fp, int *isCAS);
+int CASSETTE_Insert(const char *filename);
 void CASSETTE_Remove(void);
 extern char cassette_filename[FILENAME_MAX];
 extern char cassette_description[CASSETTE_DESCRIPTION_MAX];
@@ -23,10 +24,15 @@ extern int hold_start_on_reboot; /* preserve hold_start after reboot */
 extern int press_space;
 
 int CASSETTE_AddGap(int gaptime);
-void CASSETTE_LeaderLoad();
-void CASSETTE_LeaderSave();
+void CASSETTE_LeaderLoad(void);
+void CASSETTE_LeaderSave(void);
 int CASSETTE_Read(void);
 int CASSETTE_Write(int length);
+int CASSETTE_IOLineStatus(void);
+int CASSETTE_GetByte(void);
+int CASSETTE_GetInputIRQDelay(void);
+void CASSETTE_TapeMotor(int onoff);
+void CASSETTE_AddScanLine(void);
 extern UBYTE cassette_buffer[4096];
 
 #endif /* _CASSETTE_H_ */
