@@ -1,8 +1,8 @@
 /*
- * input.c - joystick emulation via keypad and mouse
+ * input.c - keyboard, joysticks and mouse emulation
  *
  * Copyright (C) 2001-2002 Piotr Fusik
- * Copyright (C) 2001-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2001-2006 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -22,6 +22,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "config.h"
+#include <string.h>
 #include "antic.h"
 #include "atari.h"
 #include "cassette.h"
@@ -33,9 +35,21 @@
 #include "pia.h"
 #include "platform.h"
 #include "pokey.h"
+#include "util.h"
+#ifndef CURSES_BASIC
+#include "screen.h" /* for atari_screen */
+#endif
+#ifdef __PLUS
+#include "input_win.h"
+#endif
+
+#ifdef DREAMCAST
+extern int Atari_POT(int);
+#else
+#define Atari_POT(x) 228
+#endif
 
 int key_code = AKEY_NONE;
-int key_break = 0;
 int key_shift = 0;
 int key_consol = CONSOL_NONE;
 
@@ -129,14 +143,14 @@ void INPUT_Initialise(int *argc, char *argv[])
 				mouse_mode = MOUSE_JOY;
 		}
 		else if (strcmp(argv[i], "-mouseport") == 0) {
-			mouse_port = argv[++i][0] - '1';
+			mouse_port = Util_sscandec(argv[++i]) - 1;
 			if (mouse_port < 0 || mouse_port > 3) {
 				Aprint("Invalid mouse port, using 1");
 				mouse_port = 0;
 			}
 		}
 		else if (strcmp(argv[i], "-mousespeed") == 0) {
-			mouse_speed = argv[++i][0] - '0';
+			mouse_speed = Util_sscandec(argv[++i]);
 			if (mouse_speed < 1 || mouse_speed > 9) {
 				Aprint("Invalid mouse speed, using 3");
 				mouse_speed = 3;
@@ -146,6 +160,21 @@ void INPUT_Initialise(int *argc, char *argv[])
 			joy_multijoy = 1;
 		}
 		else {
+			if (strcmp(argv[i], "-help") == 0) {
+				Aprint("\t-mouse off       Do not use mouse");
+				Aprint("\t-mouse pad       Emulate paddles");
+				Aprint("\t-mouse touch     Emulate Atari Touch Tablet");
+				Aprint("\t-mouse koala     Emulate Koala Pad");
+				Aprint("\t-mouse pen       Emulate Light Pen");
+				Aprint("\t-mouse gun       Emulate Light Gun");
+				Aprint("\t-mouse amiga     Emulate Amiga mouse");
+				Aprint("\t-mouse st        Emulate Atari ST mouse");
+				Aprint("\t-mouse trak      Emulate Atari Trak-Ball");
+				Aprint("\t-mouse joy       Emulate joystick using mouse");
+				Aprint("\t-mouseport <n>   Set mouse port 1-4 (default 1)");
+				Aprint("\t-mousespeed <n>  Set mouse speed 1-9 (default 3)");
+				Aprint("\t-multijoy        Emulate MultiJoy4 interface");
+			}
 			argv[j++] = argv[i];
 		}
 	}
@@ -280,7 +309,7 @@ void INPUT_Frame(void)
 	   (this is simply not emulated).
 	   key_code is used for keypad keys and key_shift is used for 2nd button.
 	*/
-	i = machine_type == MACHINE_5200 ? key_shift : key_break;
+	i = machine_type == MACHINE_5200 ? key_shift : (key_code == AKEY_BREAK);
 	if (i && !last_key_break) {
 		if (IRQEN & 0x80) {
 			IRQST &= ~0x80;
@@ -293,7 +322,7 @@ void INPUT_Frame(void)
 	if (key_shift)
 		SKSTAT &= ~8;
 
-	if (key_code == AKEY_NONE) {
+	if (key_code < 0) {
 		if (press_space) {
 			key_code = AKEY_SPACE;
 			press_space = 0;
@@ -302,7 +331,7 @@ void INPUT_Frame(void)
 			last_key_code = AKEY_NONE;
 		}
 	}
-	if (key_code != AKEY_NONE) {
+	if (key_code >= 0) {
 		SKSTAT &= ~4;
 		if ((key_code ^ last_key_code) & ~AKEY_SHFTCTRL) {
 		/* ignore if only shift or control has changed its state */
@@ -363,10 +392,19 @@ void INPUT_Frame(void)
 	/* handle analog joysticks in Atari 5200 */
 	if (machine_type != MACHINE_5200) {
 		for (i = 0; i < 8; i++)
-			POT_input[i] = 228;
+			POT_input[i] = Atari_POT(i);
 	}
 	else {
 		for (i = 0; i < 4; i++) {
+#ifdef DREAMCAST
+			/* first get analog js data */
+			POT_input[2 * i] = Atari_POT(2 * i);         /* x */
+			POT_input[2 * i + 1] = Atari_POT(2 * i + 1); /* y */
+			if (POT_input[2 * i] != joy_5200_center
+			 || POT_input[2 * i + 1] != joy_5200_center)
+				continue;
+			/* if analog js is unused, alternatively try keypad */
+#endif
 			if ((STICK[i] & (STICK_CENTRE ^ STICK_LEFT)) == 0)
 				POT_input[2 * i] = joy_5200_min;
 			else if ((STICK[i] & (STICK_CENTRE ^ STICK_RIGHT)) == 0)
@@ -383,6 +421,9 @@ void INPUT_Frame(void)
 	}
 
 	/* handle mouse */
+#ifdef __PLUS
+	if (g_Input.ulState & IS_CAPTURE_MOUSE)
+#endif
 	switch (mouse_mode) {
 	case MOUSE_PAD:
 	case MOUSE_TOUCH:
@@ -589,6 +630,8 @@ void INPUT_CenterMousePointer(void)
 	}
 }
 
+#ifndef CURSES_BASIC
+
 #define PLOT(dx, dy)	do {\
 							ptr[(dx) + ATARI_WIDTH * (dy)] ^= 0x0f0f;\
 							ptr[(dx) + ATARI_WIDTH * (dy) + ATARI_WIDTH / 2] ^= 0x0f0f;\
@@ -619,3 +662,5 @@ void INPUT_DrawMousePointer(void)
 		}
 	}
 }
+
+#endif /* CURSES_BASIC */
