@@ -2,7 +2,7 @@
  * mzpokeysnd.c - POKEY sound chip emulation, v1.6
  *
  * Copyright (C) 2002 Michael Borisov
- * Copyright (C) 2002-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2002-2005 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -22,22 +22,17 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "config.h"
 #include <stdlib.h>
 #include <math.h>
 
-#include "pokeysnd.h"
+#ifdef ASAP /* external project, see http://asap.sf.net */
+#include "asap_internal.h"
+#else
 #include "atari.h"
-#ifndef __PLUS
-#include "sndsave.h"
-#endif /*__PLUS*/
-#include "config.h"
-
-#include "rt-config.h"	/* extern for console_sound_enabled and serio_sound_enabled */
-
-#ifdef __PLUS
-#include "sound_win.h"
-#endif /*__PLUS*/
-
+#endif
+#include "mzpokeysnd.h"
+#include "pokeysnd.h"
 #include "remez.h"
 
 #define SND_FILTER_SIZE  2048
@@ -110,10 +105,10 @@ typedef void (*event_t)(struct stPokeyState* ps, char p5v, char p4v, char p917v)
 typedef struct stPokeyState
 {
     /* Poly positions */
-    unsigned char poly4pos;
-    unsigned char poly5pos;
-    unsigned long poly17pos;
-    unsigned long poly9pos;
+    unsigned int poly4pos;
+    unsigned int poly5pos;
+    unsigned int poly17pos;
+    unsigned int poly9pos;
 
     /* Change queue */
     unsigned char ovola;
@@ -487,44 +482,41 @@ static void build_poly4(void)
 
 static void build_poly5(void)
 {
-    unsigned char c;
-    unsigned char i;
-    unsigned char poly5=1;
+	unsigned char c;
+	unsigned char i;
+	unsigned char poly5 = 1;
 
-    for(i=0; i<31; i++)
-    {
-        poly5tbl[i] = ~poly5; /* Inversion! Attention! */
-        c = ((poly5>>2)&1) ^ ((poly5>>4)&1);
-        poly5 = ((poly5<<1)&31) + c;
-    }
+	for(i = 0; i < 31; i++) {
+		poly5tbl[i] = ~poly5; /* Inversion! Attention! */
+		c = ((poly5 >> 2) ^ (poly5 >> 4)) & 1;
+		poly5 = ((poly5 << 1) & 31) + c;
+	}
 }
 
 static void build_poly17(void)
 {
-    unsigned char c;
-    unsigned long i;
-    unsigned long poly17=1;
+	unsigned int c;
+	unsigned int i;
+	unsigned int poly17 = 1;
 
-    for(i=0; i<131071; i++)
-    {
-        poly17tbl[i] = (unsigned char)(poly17 & 0xFF);
-        c = ((poly17>>11)&1) ^ ((poly17>>16)&1);
-        poly17 = ((poly17<<1)&131071)+c;
-    }
+	for(i = 0; i < 131071; i++) {
+		poly17tbl[i] = (unsigned char) poly17;
+		c = ((poly17 >> 11) ^ (poly17 >> 16)) & 1;
+		poly17 = ((poly17 << 1) & 131071) + c;
+	}
 }
 
 static void build_poly9(void)
 {
-    unsigned char c;
-    unsigned short i;
-    unsigned short poly9 = 1;
+	unsigned int c;
+	unsigned int i;
+	unsigned int poly9 = 1;
 
-    for(i=0; i<511; i++)
-    {
-        poly9tbl[i] = (unsigned char)(poly9 & 0xFF);
-        c = ((poly9>>3)&1) ^ ((poly9>>8)&1);
-        poly9 = ((poly9<<1)&511)+c;
-    }
+	for(i = 0; i < 511; i++) {
+		poly9tbl[i] = (unsigned char) poly9;
+		c = ((poly9 >> 3) ^ (poly9 >> 8)) & 1;
+		poly9 = ((poly9 << 1) & 511) + c;
+	}
 }
 
 static void advance_polies(PokeyState* ps, unsigned long tacts)
@@ -968,21 +960,21 @@ static double generate_sample(PokeyState* ps)
  filter table generator by Krzysztof Nikiel
  ******************************************/
 
-static int remez_filter_table(double resamp_rate, // output_rate/input_rate
+static int remez_filter_table(double resamp_rate, /* output_rate/input_rate */
                               double *cutoff, int quality)
 {
   int i;
   static const int orders[] = {600, 800, 1000, 1200};
   static const struct {
-    int stop;		// stopband ripple
-    double weight;	// stopband weight
+    int stop;		/* stopband ripple */
+    double weight;	/* stopband weight */
     double twidth[sizeof(orders)/sizeof(orders[0])];
   } paramtab[] =
   {
     {70, 90, {4.9e-3, 3.45e-3, 2.65e-3, 2.2e-3}},
     {55, 25, {3.4e-3, 2.7e-3, 2.05e-3, 1.7e-3}},
     {40, 6.0, {2.6e-3, 1.8e-3, 1.5e-3, 1.2e-3}},
-    {-1}
+    {-1, 0, {0, 0, 0, 0}}
   };
   static const double passtab[] = {0.5, 0.6, 0.7};
   int ripple = 0, order = 0;
@@ -993,21 +985,21 @@ static int remez_filter_table(double resamp_rate, // output_rate/input_rate
 
   *cutoff = 0.95 * 0.5 * resamp_rate;
 
-  if (quality >= sizeof(passtab)/sizeof(passtab[0]))
-    quality = sizeof(passtab)/sizeof(passtab[0]) - 1;
+  if (quality >= (int) (sizeof(passtab) / sizeof(passtab[0])))
+    quality = (int) (sizeof(passtab) / sizeof(passtab[0])) - 1;
 
   for (ripple = 0; paramtab[ripple].stop > 0; ripple++)
   {
-    for (order = 0; order < (sizeof(orders)/sizeof(orders[0])); order++)
+    for (order = 0; order < (int) (sizeof(orders)/sizeof(orders[0])); order++)
     {
       if ((*cutoff - paramtab[ripple].twidth[order])
 	  > passtab[quality] * 0.5 * resamp_rate)
-	// transition width OK
+	/* transition width OK */
 	goto found;
     }
   }
 
-  // not found -- use shortest trasition
+  /* not found -- use shortest trasition */
   ripple--;
   order--;
 
@@ -1024,7 +1016,7 @@ found:
 
   size = orders[order] + 1;
 
-  if (size > SND_FILTER_SIZE) // static table too short
+  if (size > SND_FILTER_SIZE) /* static table too short */
     return 0;
 
   desired[0] = 1;
@@ -1054,7 +1046,7 @@ found:
     }
   }
 
-  // compute reversed cumulative sum table
+  /* compute reversed cumulative sum table */
   for (i = size - 2; i >= 0; i--)
     filter_data[i] += filter_data[i + 1];
 
@@ -1100,7 +1092,11 @@ static void Update_vol_only_sound_mz( void );
 #endif
 
 int Pokey_sound_init_mz(uint32 freq17, uint16 playback_freq, uint8 num_pokeys,
-			int flags, int quality)
+                        int flags, int quality
+#ifdef __PLUS
+                        , int clear_regs
+#endif
+                       )
 {
     double cutoff;
 
@@ -1123,10 +1119,7 @@ int Pokey_sound_init_mz(uint32 freq17, uint16 playback_freq, uint8 num_pokeys,
 	samp_freq=playback_freq;
 #endif  /* VOL_ONLY_SOUND */
 
-    if (flags & SND_BIT16)
-      Pokey_process = Pokey_process_16;
-    else
-      Pokey_process = Pokey_process_8;
+	Pokey_process_ptr = (flags & SND_BIT16) ? Pokey_process_16 : Pokey_process_8;
 
     switch(playback_freq)
     {
@@ -1207,7 +1200,7 @@ int Pokey_sound_init_mz(uint32 freq17, uint16 playback_freq, uint8 num_pokeys,
           * sample_rate;
 	filter_size = remez_filter_table((double)sample_rate/pokey_frq,
 					 &cutoff, quality);
-	audible_frq = cutoff * pokey_frq;
+	audible_frq = (unsigned) (cutoff * pokey_frq);
     }
 
     build_poly4();
@@ -1215,11 +1208,16 @@ int Pokey_sound_init_mz(uint32 freq17, uint16 playback_freq, uint8 num_pokeys,
     build_poly9();
     build_poly17();
 
-    ResetPokeyState(pokey_states);
-    ResetPokeyState(pokey_states+1);
-    num_cur_pokeys = num_pokeys;
+#ifdef __PLUS
+	if (clear_regs)
+#endif
+	{
+		ResetPokeyState(pokey_states);
+		ResetPokeyState(pokey_states + 1);
+	}
+	num_cur_pokeys = num_pokeys;
 
-    return 0; /* OK */
+	return 0; /* OK */
 }
 
 /*****************************************************************************/
@@ -1464,10 +1462,10 @@ static void Update_audctl(PokeyState* ps, unsigned char val)
     unsigned char recalc2=0;
     unsigned char recalc3=0;
 
-    unsigned char cnt0=0;
-    unsigned char cnt1=0;
-    unsigned char cnt2=0;
-    unsigned char cnt3=0;
+    unsigned int cnt0 = 0;
+    unsigned int cnt1 = 0;
+    unsigned int cnt2 = 0;
+    unsigned int cnt3 = 0;
 
     nc0_hf = (val & 0x40) != 0;
     nc2_hf = (val & 0x20) != 0;
@@ -1886,7 +1884,7 @@ static void Update_pokey_sound_mz(uint16 addr, uint8 val, uint8 chip, uint8 gain
 void Pokey_debugreset(uint8 chip)
 {
     PokeyState* ps = pokey_states+chip;
-    
+
     if(ps->c1_f0)
         ps->c0divpos = ps->c0divstart_p;
     else
@@ -1904,21 +1902,6 @@ void Pokey_debugreset(uint8 chip)
     ps->c3t2 = 1;
 }
 
-/*****************************************************************************/
-/* Module:  Pokey_process()                                                  */
-/* Purpose: To fill the output buffer with the sound output based on the     */
-/*          pokey chip parameters.                                           */
-/*                                                                           */
-/* Inputs:  *buffer - pointer to the buffer where the audio output will      */
-/*                    be placed                                              */
-/*          n - size of the playback buffer                                  */
-/*          num_pokeys - number of currently active pokeys to process        */
-/*                                                                           */
-/* Outputs: the buffer will be filled with n bytes of audio - no return val  */
-/*          Also the buffer will be written to disk if Sound recording is ON */
-/*                                                                           */
-/*****************************************************************************/
-
 /**************************************************************
 
            Master gain and DC offset calculation
@@ -2000,7 +1983,7 @@ void Pokey_debugreset(uint8 chip)
  volume seems to be pretty much like 8 on single POKEY's channel.
  So, the volumes now can sum up to 136 (4 channels * 15 * 2
  + 8 * 2 for GTIA), not 120.
- 
+
  A note from Mark Grebe:
  I've added back in the console and sio sounds from the old
  pokey version.  So, now the volumes can sum up to 152
@@ -2092,14 +2075,14 @@ static void Pokey_process_8(void* sndbuffer, unsigned sndn)
 {
     unsigned short i;
     int nsam = sndn;
-    uint8 *buffer = sndbuffer;
+    uint8 *buffer = (uint8 *) sndbuffer;
 
     if(num_cur_pokeys<1)
         return; /* module was not initialized */
 
     /* if there are two pokeys, then the signal is stereo
        we assume even sndn */
-    while(nsam>=num_cur_pokeys)
+    while(nsam >= (int) num_cur_pokeys)
     {
 #ifdef VOL_ONLY_SOUND
         if( sampbuf_rptr!=sampbuf_ptr )
@@ -2112,7 +2095,7 @@ static void Pokey_process_8(void* sndbuffer, unsigned sndn)
                         if( sampbuf_rptr>=SAMPBUF_MAX )
                                 sampbuf_rptr=0;
                         if( sampbuf_rptr!=sampbuf_ptr )
-                            {   
+                            {
                             sampbuf_cnt[sampbuf_rptr]+=l;
                             }
                         else	break;
@@ -2141,14 +2124,14 @@ static void Pokey_process_16(void* sndbuffer, unsigned sndn)
 {
     unsigned short i;
     int nsam = sndn;
-    int16 *buffer = sndbuffer;
+    int16 *buffer = (int16 *) sndbuffer;
 
     if(num_cur_pokeys<1)
         return; /* module was not initialized */
 
     /* if there are two pokeys, then the signal is stereo
        we assume even sndn */
-    while(nsam>=num_cur_pokeys)
+    while(nsam >= (int) num_cur_pokeys)
     {
 #ifdef VOL_ONLY_SOUND
         if( sampbuf_rptr!=sampbuf_ptr )
@@ -2161,20 +2144,20 @@ static void Pokey_process_16(void* sndbuffer, unsigned sndn)
                         if( sampbuf_rptr>=SAMPBUF_MAX )
                                 sampbuf_rptr=0;
                         if( sampbuf_rptr!=sampbuf_ptr )
-                            {   
+                            {
                             sampbuf_cnt[sampbuf_rptr]+=l;
                             }
                         else	break;
                 }
             }
-#endif 
+#endif
 #ifdef VOL_ONLY_SOUND
         buffer[0] = (int16)floor((generate_sample(pokey_states) + sampout - MAX_SAMPLE / 2.0)
          * (65535.0 / MAX_SAMPLE / 4 * M_PI * 0.95) + 0.5 + 0.5 * rand() / RAND_MAX - 0.25);
 #else
         buffer[0] = (int16)floor((generate_sample(pokey_states) - MAX_SAMPLE / 2.0)
          * (65535.0 / MAX_SAMPLE / 4 * M_PI * 0.95) + 0.5 + 0.5 * rand() / RAND_MAX - 0.25);
-#endif         
+#endif
         for(i=1; i<num_cur_pokeys; i++)
         {
             buffer[i] = (int16)floor((generate_sample(pokey_states + i) - MAX_SAMPLE / 2.0)
@@ -2191,7 +2174,7 @@ static void Update_serio_sound_mz( int out, UBYTE data )
 #ifdef VOL_ONLY_SOUND
    int bits,pv,future;
         if (!serio_sound_enabled) return;
-  
+
 	pv=0;
 	future=0;
 	bits= (data<<1) | 0x200;
@@ -2224,13 +2207,13 @@ static void Update_serio_sound_mz( int out, UBYTE data )
 
 #ifdef CONSOLE_SOUND
 static void Update_consol_sound_mz( int set )
-{ 
+{
 #ifdef VOL_ONLY_SOUND
   static int prev_atari_speaker=0;
   static unsigned int prev_cpu_clock=0;
   int d;
         if (!console_sound_enabled) return;
-  
+
 	if( !set && samp_consol_val==0 )	return;
 	sampbuf_lastval-=samp_consol_val;
 	if( prev_atari_speaker!=atari_speaker )
@@ -2274,50 +2257,3 @@ static void Update_vol_only_sound_mz( void )
 #endif /* CONSOLE_SOUND */
 }
 #endif
-
-/*****************************************************************************
-
-  REVISION HISTORY
-
-$Log: mzpokeysnd.c,v $
-Revision 1.15  2003/12/12 00:23:24  markgrebe
-Added console and sio Sound back in
-
-Revision 1.14  2003/10/25 18:40:54  joy
-various little updates for better MacOSX support
-
-Revision 1.13  2003/07/04 19:12:17  pfusik
-added key click
-
-Revision 1.12  2003/03/03 09:57:33  joy
-Ed improved autoconf again plus fixed some little things
-
-Revision 1.11  2003/02/24 09:33:05  joy
-header cleanup
-
-Revision 1.10  2003/02/19 14:07:47  joy
-configure stuff cleanup
-
-Revision 1.9  2003/02/17 17:11:17  knik
-remez_filter_table(): fixed static constants and variables
-
-Revision 1.8  2003/02/12 05:13:18  vasyl
-Corrected minor deviation from standard C
-
-Revision 1.7  2003/01/06 17:11:09  knik
-used new equiripple filter design algorithm (remez.c)
-
-
-
-  V1.5  11.12.2002   improved gain and dc offset values
-                     corrected STIMER bug and 16-bit bug
-                     added dithering
-  V1.3  04.12.2002   All C++ style comments changed to C style for
-                      portability
-                     poly17tbl and poly9tbl converted to unsigned char
-                      to save memory
-                     copyright info removed from static code
-                      also to save some memory - M.B.
-  V1.2  02.12.2002   Added STIMER feature - M.B.
-
- *****************************************************************************/
