@@ -2,7 +2,7 @@
  * pokeysnd.c - POKEY sound chip emulation, v2.4
  *
  * Copyright (C) 1996-1998 Ron Fries
- * Copyright (C) 1998-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 1998-2006 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -22,104 +22,46 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/*
-   Revision History:
-
-   09/22/96 - Ron Fries - Initial Release
-   01/14/97 - Ron Fries - Corrected a minor problem to improve sound quality
-                          Also changed names from POKEY11.x to POKEY.x
-   01/17/97 - Ron Fries - Added support for multiple POKEY chips.
-   03/31/97 - Ron Fries - Made some minor mods for MAME (changed to signed
-                          8-bit sample, increased gain range, removed
-                          _disable() and _enable().)
-   04/06/97 - Brad Oliver - Some cross-platform modifications. Added
-                            big/little endian #defines, removed <dos.h>,
-                            conditional defines for TRUE/FALSE
-   01/19/98 - Ron Fries - Changed signed/unsigned sample support to a
-                          compile-time option.  Defaults to unsigned -
-                          define SIGNED_SAMPLES to create signed.
-   03/22/98 - Ron Fries - Added 'filter' support to channels 1 & 2.
-   09/29/99 - Krzysztof Nikiel - Changed Pokey_process main loop;
-                                 added output interpolation to reduce
-                                 DSP interference
-
-   V2.0 Detailed Changes
-   ---------------------
-
-   Now maintains both a POLY9 and POLY17 counter.  Though this slows the
-   emulator in general, it was required to support mutiple POKEYs since
-   each chip can individually select POLY9 or POLY17 operation.  Also,
-   eliminated the Poly17_size variable.
-
-   Changed address of POKEY chip.  In the original, the chip was fixed at
-   location D200 for compatibility with the Atari 800 line of 8-bit
-   computers. The update function now only examines the lower four bits, so
-   the location for all emulated chips is effectively xxx0 - xxx8.
-
-   The Update_pokey_sound function has two additional parameters which
-   selects the desired chip and selects the desired gain.
-
-   Added clipping to reduce distortion, configurable at compile-time.
-
-   The Pokey_sound_init function has an additional parameter which selects
-   the number of pokey chips to emulate.
-
-   The output will be amplified by gain/16.  If the output exceeds the
-   maximum value after the gain, it will be limited to reduce distortion.
-   The best value for the gain depends on the number of POKEYs emulated
-   and the maximum volume used.  The maximum possible output for each
-   channel is 15, making the maximum possible output for a single chip to
-   be 60.  Assuming all four channels on the chip are used at full volume,
-   a gain of 64 can be used without distortion.  If 4 POKEY chips are
-   emulated and all 16 channels are used at full volume, the gain must be
-   no more than 16 to prevent distortion.  Of course, if only a few of the
-   16 channels are used or not all channels are used at full volume, a
-   larger gain can be used.
-
-   The Pokey_process routine automatically processes and mixes all selected
-   chips/channels.  No additional calls or functions are required.
-
-   The unoptimized Pokey_process2() function has been removed.
-*/
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-#include <limits.h>
-
-#include "pokeysnd.h"
-#include "mzpokeysnd.h"
-#include "atari.h"
-#include "sndsave.h"
-#include "rt-config.h"	/* extern for stereo_enabled and enable_new_pokey */
-
 #include "config.h"
 
-#ifdef UNALIGNED_LONG_OK
-#  define READ_U32(x)    (*(uint32 *)(x))
-#  define WRITE_U32(x,d) (*(uint32 *)(x)=(d))
+#ifdef ASAP /* external project, see http://asap.sf.net */
+#include "asap_internal.h"
+#else
+#include "atari.h"
+#ifndef __PLUS
+#include "sndsave.h"
+#else
+#include "sound_win.h"
+#endif
+#endif
+#include "mzpokeysnd.h"
+#include "pokeysnd.h"
+
+#ifdef WORDS_UNALIGNED_OK
+#  define READ_U32(x)     (*(uint32 *) (x))
+#  define WRITE_U32(x, d) (*(uint32 *) (x) = (d))
 #else
 #  ifdef WORDS_BIGENDIAN
-#    define READ_U32(x) (((*(unsigned char *)(x)) << 24) | ((*((unsigned char *)(x)+1)) << 16) | \
-                        ((*((unsigned char *)(x)+2)) << 8) | ((*((unsigned char *)(x)+3))))
-#    define WRITE_U32(x,d) \
+#    define READ_U32(x) (((*(unsigned char *)(x)) << 24) | ((*((unsigned char *)(x) + 1)) << 16) | \
+                        ((*((unsigned char *)(x) + 2)) << 8) | ((*((unsigned char *)(x) + 3))))
+#    define WRITE_U32(x, d) \
   { \
   uint32 i = d; \
-  (*(unsigned char *)(x)) = (((i)>>24)&255); \
-  (*((unsigned char *)(x)+1)) = (((i)>>16)&255); \
-  (*((unsigned char *)(x)+2)) = (((i)>>8)&255); \
-  (*((unsigned char *)(x)+3)) = ((i)&255); \
+  (*(unsigned char *) (x)) = (((i) >> 24) & 255); \
+  (*((unsigned char *) (x) + 1)) = (((i) >> 16) & 255); \
+  (*((unsigned char *) (x) + 2)) = (((i) >> 8) & 255); \
+  (*((unsigned char *) (x) + 3)) = ((i) & 255); \
   }
 #  else
-#    define READ_U32(x) ((*(unsigned char *)(x)) | ((*((unsigned char *)(x)+1)) << 8) | \
-                        ((*((unsigned char *)(x)+2)) << 16) | ((*((unsigned char *)(x)+3)) << 24))
-#    define WRITE_U32(x,d) \
+#    define READ_U32(x) ((*(unsigned char *) (x)) | ((*((unsigned char *) (x) + 1)) << 8) | \
+                        ((*((unsigned char *) (x) + 2)) << 16) | ((*((unsigned char *) (x) + 3)) << 24))
+#    define WRITE_U32(x, d) \
   { \
   uint32 i = d; \
-  (*(unsigned char *)(x)) = ((i)&255); \
-  (*((unsigned char *)(x)+1)) = (((i)>>8)&255); \
-  (*((unsigned char *)(x)+2)) = (((i)>>16)&255); \
-  (*((unsigned char *)(x)+3)) = (((i)>>24)&255); \
+  (*(unsigned char *)(x)) = ((i) & 255); \
+  (*((unsigned char *)(x) + 1)) = (((i) >> 8) & 255); \
+  (*((unsigned char *)(x) + 2)) = (((i) >> 16) & 255); \
+  (*((unsigned char *)(x) + 3)) = (((i) >> 24) & 255); \
   }
 #  endif
 #endif
@@ -188,8 +130,8 @@ int	sampbuf_last = 0;		/* last absolute time */
 int	sampbuf_AUDV[4 * MAXPOKEYS];	/* prev. channel volume */
 int	sampbuf_lastval = 0;		/* last volume */
 int	sampout;			/* last out volume */
-uint16	samp_freq;
-int	samp_consol_val=0;		/* actual value of console sound */
+uint16 samp_freq;
+int	samp_consol_val = 0;		/* actual value of console sound */
 #ifdef STEREO_SOUND
 int	sampbuf_val2[SAMPBUF_MAX];	/* volume values */
 int	sampbuf_cnt2[SAMPBUF_MAX];	/* relative start time */
@@ -202,16 +144,24 @@ int	sampout2;			/* last out volume */
 #endif  /* VOL_ONLY_SOUND */
 
 static uint32 snd_freq17 = FREQ_17_EXACT;
-static uint16 snd_playback_freq = 44100;
-static uint8 snd_num_pokeys = 1;
+int32 snd_playback_freq = 44100;
+uint8 snd_num_pokeys = 1;
 static int snd_flags = 0;
 static int mz_quality = 0;		/* default quality for mzpokeysnd */
+#ifdef __PLUS
+int mz_clear_regs = 0;
+#endif
+
+int enable_new_pokey = TRUE;
+#ifndef ASAP
+int stereo_enabled = FALSE;
+#endif
 
 /* multiple sound engine interface */
-static void Pokey_process_8(void * sndbuffer, unsigned sndn);
-static void Pokey_process_16(void * sndbuffer, unsigned sndn);
+static void Pokey_process_8(void *sndbuffer, unsigned sndn);
+static void Pokey_process_16(void *sndbuffer, unsigned sndn);
 static void null_pokey_process(void *sndbuffer, unsigned int sndn) {}
-void (* Pokey_process)(void * sndbuffer, unsigned int sndn) = null_pokey_process;
+void (*Pokey_process_ptr)(void *sndbuffer, unsigned int sndn) = null_pokey_process;
 
 static void Update_pokey_sound_rf(uint16, uint8, uint8, uint8);
 static void null_pokey_sound(uint16 addr, uint8 val, uint8 chip, uint8 gain) {}
@@ -222,17 +172,21 @@ void (*Update_pokey_sound) (uint16 addr, uint8 val, uint8 chip, uint8 gain)
 static void Update_serio_sound_rf(int out, UBYTE data);
 static void null_serio_sound(int out, UBYTE data) {}
 void (*Update_serio_sound)(int out, UBYTE data) = null_serio_sound;
-#endif
 int serio_sound_enabled = 1;
+#endif
 
+#ifdef CONSOLE_SOUND
 static void Update_consol_sound_rf(int set);
 static void null_consol_sound(int set) {}
 void (*Update_consol_sound)(int set) = null_consol_sound;
 int console_sound_enabled = 1;
+#endif
 
+#ifdef VOL_ONLY_SOUND
 static void Update_vol_only_sound_rf(void);
 static void null_vol_only_sound(void) {}
 void (*Update_vol_only_sound)(void) = null_vol_only_sound;
+#endif
 
 /*****************************************************************************/
 /* In my routines, I treat the sample output as another divide by N counter  */
@@ -280,23 +234,34 @@ void (*Update_vol_only_sound)(void) = null_vol_only_sound;
 static int Pokey_sound_init_rf(uint32 freq17, uint16 playback_freq,
            uint8 num_pokeys, unsigned int flags);
 
-int Pokey_DoInit()
+int Pokey_DoInit(void)
 {
 	if (enable_new_pokey)
-		return Pokey_sound_init_mz(snd_freq17, snd_playback_freq,
-				snd_num_pokeys, snd_flags, mz_quality);
+		return Pokey_sound_init_mz(snd_freq17, (uint16) snd_playback_freq,
+				snd_num_pokeys, snd_flags, mz_quality
+#ifdef __PLUS
+				, mz_clear_regs
+#endif
+		);
 	else
-		return Pokey_sound_init_rf(snd_freq17, snd_playback_freq,
+		return Pokey_sound_init_rf(snd_freq17, (uint16) snd_playback_freq,
 				snd_num_pokeys, snd_flags);
 }
 
 int Pokey_sound_init(uint32 freq17, uint16 playback_freq, uint8 num_pokeys,
-		     unsigned int flags)
+                     unsigned int flags
+#ifdef __PLUS
+                     , int clear_regs
+#endif
+)
 {
 	snd_freq17 = freq17;
 	snd_playback_freq = playback_freq;
 	snd_num_pokeys = num_pokeys;
 	snd_flags = flags;
+#ifdef __PLUS
+	mz_clear_regs = clear_regs;
+#endif
 
 	return Pokey_DoInit();
 }
@@ -306,6 +271,14 @@ void Pokey_set_mzquality(int quality)	/* specially for win32, perhaps not needed
 	mz_quality = quality;
 }
 
+void Pokey_process(void *sndbuffer, unsigned int sndn)
+{
+	Pokey_process_ptr(sndbuffer, sndn);
+#if !defined(__PLUS) && !defined(ASAP)
+	WriteToSoundFile(sndbuffer, sndn);
+#endif
+}
+
 static int Pokey_sound_init_rf(uint32 freq17, uint16 playback_freq,
            uint8 num_pokeys, unsigned int flags)
 {
@@ -322,17 +295,11 @@ static int Pokey_sound_init_rf(uint32 freq17, uint16 playback_freq,
 	Update_vol_only_sound = Update_vol_only_sound_rf;
 #endif
 
-	if (flags & SND_BIT16)
-	  Pokey_process = Pokey_process_16;
-        else
-	  Pokey_process = Pokey_process_8;
+	Pokey_process_ptr = (flags & SND_BIT16) ? Pokey_process_16 : Pokey_process_8;
 
 #ifdef VOL_ONLY_SOUND
-	samp_freq=playback_freq;
-#endif  /* VOL_ONLY_SOUND */
-
-	/* disable interrupts to handle critical sections */
-	/*    _disable(); */ /* RSF - removed for portability 31-MAR-97 */
+	samp_freq = playback_freq;
+#endif
 
 	/* start all of the polynomial counters at zero */
 	P4 = 0;
@@ -360,9 +327,7 @@ static int Pokey_sound_init_rf(uint32 freq17, uint16 playback_freq,
 	/* set the number of pokey chips currently emulated */
 	Num_pokeys = num_pokeys;
 
-	/*    _enable(); */ /* RSF - removed for portability 31-MAR-97 */
-
-	return 0; // OK
+	return 0; /* OK */
 }
 
 
@@ -393,9 +358,6 @@ static void Update_pokey_sound_rf(uint16 addr, uint8 val, uint8 chip,
 	uint8 chan_mask;
 	uint8 chip_offs;
 
-	/* disable interrupts to handle critical sections */
-	/*    _disable(); */ /* RSF - removed for portability 31-MAR-97 */
-
 	/* calculate the chip_offs for the channel arrays */
 	chip_offs = chip << 2;
 
@@ -404,75 +366,59 @@ static void Update_pokey_sound_rf(uint16 addr, uint8 val, uint8 chip,
 	case _AUDF1:
 		/* AUDF[CHAN1 + chip_offs] = val; */
 		chan_mask = 1 << CHAN1;
-
 		if (AUDCTL[chip] & CH1_CH2)		/* if ch 1&2 tied together */
 			chan_mask |= 1 << CHAN2;	/* then also change on ch2 */
 		break;
-
 	case _AUDC1:
 		/* AUDC[CHAN1 + chip_offs] = val; */
-		/* RSF - changed gain (removed >> 4) 31-MAR-97 */
 		AUDV[CHAN1 + chip_offs] = (val & VOLUME_MASK) * gain;
 		chan_mask = 1 << CHAN1;
 		break;
-
 	case _AUDF2:
 		/* AUDF[CHAN2 + chip_offs] = val; */
 		chan_mask = 1 << CHAN2;
 		break;
-
 	case _AUDC2:
 		/* AUDC[CHAN2 + chip_offs] = val; */
-		/* RSF - changed gain (removed >> 4) 31-MAR-97 */
 		AUDV[CHAN2 + chip_offs] = (val & VOLUME_MASK) * gain;
 		chan_mask = 1 << CHAN2;
 		break;
-
 	case _AUDF3:
 		/* AUDF[CHAN3 + chip_offs] = val; */
 		chan_mask = 1 << CHAN3;
-
 		if (AUDCTL[chip] & CH3_CH4)		/* if ch 3&4 tied together */
 			chan_mask |= 1 << CHAN4;	/* then also change on ch4 */
 		break;
-
 	case _AUDC3:
 		/* AUDC[CHAN3 + chip_offs] = val; */
-		/* RSF - changed gain (removed >> 4) 31-MAR-97 */
 		AUDV[CHAN3 + chip_offs] = (val & VOLUME_MASK) * gain;
 		chan_mask = 1 << CHAN3;
 		break;
-
 	case _AUDF4:
 		/* AUDF[CHAN4 + chip_offs] = val; */
 		chan_mask = 1 << CHAN4;
 		break;
-
 	case _AUDC4:
 		/* AUDC[CHAN4 + chip_offs] = val; */
-		/* RSF - changed gain (removed >> 4) 31-MAR-97 */
 		AUDV[CHAN4 + chip_offs] = (val & VOLUME_MASK) * gain;
 		chan_mask = 1 << CHAN4;
 		break;
-
 	case _AUDCTL:
 		/* AUDCTL[chip] = val; */
 		chan_mask = 15;			/* all channels */
-
 		break;
-
 	default:
 		chan_mask = 0;
 		break;
 	}
 
-/************************************************************/
+	/************************************************************/
 	/* As defined in the manual, the exact Div_n_cnt values are */
 	/* different depending on the frequency and resolution:     */
 	/*    64 kHz or 15 kHz - AUDF + 1                           */
 	/*    1 MHz, 8-bit -     AUDF + 4                           */
 	/*    1 MHz, 16-bit -    AUDF[CHAN1]+256*AUDF[CHAN2] + 7    */
-/************************************************************/
+	/************************************************************/
 
 	/* only reset the channels that have changed */
 
@@ -555,53 +501,63 @@ static void Update_pokey_sound_rf(uint16 addr, uint8 val, uint8 chip,
 	/* if channel is volume only, set current output */
 	for (chan = CHAN1; chan <= CHAN4; chan++) {
 		if (chan_mask & (1 << chan)) {
-#ifdef VOL_ONLY_SOUND
-			if( (AUDC[chan + chip_offs] & VOL_ONLY) )
-			{
-#ifdef STEREO_SOUND
-				if( chip&0x01 )
-				{
-				sampbuf_lastval2+=AUDV[chan + chip_offs]
-					-sampbuf_AUDV[chan + chip_offs];
 
-				sampbuf_val2[sampbuf_ptr2]=sampbuf_lastval2;
-				sampbuf_AUDV[chan + chip_offs]=AUDV[chan + chip_offs];
-				sampbuf_cnt2[sampbuf_ptr2]=
-					(cpu_clock-sampbuf_last2)*128*samp_freq/178979;
-				sampbuf_last2=cpu_clock;
-				sampbuf_ptr2++;
-				if( sampbuf_ptr2>=SAMPBUF_MAX )
-					sampbuf_ptr2=0;
-				if( sampbuf_ptr2==sampbuf_rptr2 )
-				{	sampbuf_rptr2++;
-					if( sampbuf_rptr2>=SAMPBUF_MAX )
-						sampbuf_rptr2=0;
-				}
+#ifdef VOL_ONLY_SOUND
+
+#ifdef __PLUS
+			if (g_Sound.nDigitized)
+#endif
+			if ((AUDC[chan + chip_offs] & VOL_ONLY)) {
+
+#ifdef STEREO_SOUND
+
+#ifdef __PLUS
+				if (stereo_enabled && chip & 0x01)
+#else
+				if (chip & 0x01)
+#endif
+				{
+					sampbuf_lastval2 += AUDV[chan + chip_offs]
+						- sampbuf_AUDV[chan + chip_offs];
+
+					sampbuf_val2[sampbuf_ptr2] = sampbuf_lastval2;
+					sampbuf_AUDV[chan + chip_offs] = AUDV[chan + chip_offs];
+					sampbuf_cnt2[sampbuf_ptr2] =
+						(cpu_clock - sampbuf_last2) * 128 * samp_freq / 178979;
+					sampbuf_last2 = cpu_clock;
+					sampbuf_ptr2++;
+					if (sampbuf_ptr2 >= SAMPBUF_MAX)
+						sampbuf_ptr2 = 0;
+					if (sampbuf_ptr2 == sampbuf_rptr2) {
+						sampbuf_rptr2++;
+						if (sampbuf_rptr2 >= SAMPBUF_MAX)
+							sampbuf_rptr2 = 0;
+					}
 				}
 				else
+#endif /* STEREO_SOUND */
 				{
-#endif
-				sampbuf_lastval+=AUDV[chan + chip_offs]
-					-sampbuf_AUDV[chan + chip_offs];
+					sampbuf_lastval += AUDV[chan + chip_offs]
+						-sampbuf_AUDV[chan + chip_offs];
 
-				sampbuf_val[sampbuf_ptr]=sampbuf_lastval;
-				sampbuf_AUDV[chan + chip_offs]=AUDV[chan + chip_offs];
-				sampbuf_cnt[sampbuf_ptr]=
-					(cpu_clock-sampbuf_last)*128*samp_freq/178979;
-				sampbuf_last=cpu_clock;
-				sampbuf_ptr++;
-				if( sampbuf_ptr>=SAMPBUF_MAX )
-					sampbuf_ptr=0;
-				if( sampbuf_ptr==sampbuf_rptr )
-				{	sampbuf_rptr++;
-					if( sampbuf_rptr>=SAMPBUF_MAX )
-						sampbuf_rptr=0;
+					sampbuf_val[sampbuf_ptr] = sampbuf_lastval;
+					sampbuf_AUDV[chan + chip_offs] = AUDV[chan + chip_offs];
+					sampbuf_cnt[sampbuf_ptr] =
+						(cpu_clock - sampbuf_last) * 128 * samp_freq / 178979;
+					sampbuf_last = cpu_clock;
+					sampbuf_ptr++;
+					if (sampbuf_ptr >= SAMPBUF_MAX)
+						sampbuf_ptr = 0;
+					if (sampbuf_ptr == sampbuf_rptr) {
+						sampbuf_rptr++;
+						if (sampbuf_rptr >= SAMPBUF_MAX)
+							sampbuf_rptr = 0;
+					}
 				}
-#ifdef STEREO_SOUND
-				}
-#endif
 			}
-#endif  /* VOL_ONLY_SOUND */
+
+#endif /* VOL_ONLY_SOUND */
+
 			/* I've disabled any frequencies that exceed the sampling
 			   frequency.  There isn't much point in processing frequencies
 			   that the hardware can't reproduce.  I've also disabled
@@ -612,7 +568,12 @@ static void Update_pokey_sound_rf(uint16 addr, uint8 val, uint8 chip,
 			/* or the channel freq is greater than the playback freq */
 			if ( (AUDC[chan + chip_offs] & VOL_ONLY) ||
 				((AUDC[chan + chip_offs] & VOLUME_MASK) == 0)
-				/* || (Div_n_max[chan + chip_offs] < (Samp_n_max >> 8))*/) {
+#if defined(__PLUS) && !defined(_WX_)
+				|| (!g_Sound.nBieniasFix && (Div_n_max[chan + chip_offs] < (Samp_n_max >> 8)))
+#else
+				/* || (Div_n_max[chan + chip_offs] < (Samp_n_max >> 8))*/
+#endif
+				) {
 				/* indicate the channel is 'on' */
 				Outvol[chan + chip_offs] = 1;
 
@@ -621,7 +582,12 @@ static void Update_pokey_sound_rf(uint16 addr, uint8 val, uint8 chip,
 					(chan == CHAN4 && !(AUDCTL[chip] & CH2_FILTER)) ||
 					(chan == CHAN1) ||
 					(chan == CHAN2)
-					/* || (Div_n_max[chan + chip_offs] < (Samp_n_max >> 8))*/) {
+#if defined(__PLUS) && !defined(_WX_)
+					|| (!g_Sound.nBieniasFix && (Div_n_max[chan + chip_offs] < (Samp_n_max >> 8)))
+#else
+					/* || (Div_n_max[chan + chip_offs] < (Samp_n_max >> 8))*/
+#endif
+				) {
 					/* and set channel freq to max to reduce processing */
 					Div_n_max[chan + chip_offs] = 0x7fffffffL;
 					Div_n_cnt[chan + chip_offs] = 0x7fffffffL;
@@ -652,9 +618,9 @@ static void Update_pokey_sound_rf(uint16 addr, uint8 val, uint8 chip,
 /*                                                                           */
 /*****************************************************************************/
 
-void Pokey_process_8(void * sndbuffer, unsigned sndn)
+static void Pokey_process_8(void *sndbuffer, unsigned sndn)
 {
-	register uint8 *buffer = sndbuffer;
+	register uint8 *buffer = (uint8 *) sndbuffer;
 	register uint16 n = sndn;
 
 	register uint32 *div_n_ptr;
@@ -666,12 +632,12 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 #ifdef STEREO_SOUND
 	register int16 cur_val2;
 #endif
-#else
+#else /* CLIP_SOUND */
 	register uint8 cur_val;		/* otherwise we'll simplify as 8-bit unsigned */
 #ifdef STEREO_SOUND
-	register int8 cur_val2;
-#endif
+	register uint8 cur_val2;
 #endif
+#endif /* CLIP_SOUND */
 	register uint8 *out_ptr;
 	register uint8 audc;
 	register uint8 toggle;
@@ -694,8 +660,11 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 	/* add the output values of all 4 channels */
 	cur_val = SAMP_MIN;
 #ifdef STEREO_SOUND
-	cur_val2 = SAMP_MIN;
+#ifdef __PLUS
+	if (stereo_enabled)
 #endif
+	cur_val2 = SAMP_MIN;
+#endif /* STEREO_SOUND */
 
 	count = Num_pokeys;
 	do {
@@ -715,31 +684,39 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 			cur_val += *vol_ptr;
 		vol_ptr++;
 #ifdef STEREO_SOUND
-		count--;
-		if( count ) {
-			if (*out_ptr++)
-				cur_val2 += *vol_ptr;
-			vol_ptr++;
-
-			if (*out_ptr++)
-				cur_val2 += *vol_ptr;
-			vol_ptr++;
-
-			if (*out_ptr++)
-				cur_val2 += *vol_ptr;
-			vol_ptr++;
-
-			if (*out_ptr++)
-				cur_val2 += *vol_ptr;
-			vol_ptr++;
-		}
-		else	break;
+#ifdef __PLUS
+		if (stereo_enabled)
 #endif
+		{
+			count--;
+			if (count) {
+				if (*out_ptr++)
+					cur_val2 += *vol_ptr;
+				vol_ptr++;
+
+				if (*out_ptr++)
+					cur_val2 += *vol_ptr;
+				vol_ptr++;
+
+				if (*out_ptr++)
+					cur_val2 += *vol_ptr;
+				vol_ptr++;
+
+				if (*out_ptr++)
+					cur_val2 += *vol_ptr;
+				vol_ptr++;
+			}
+			else
+				break;
+		}
+#endif /* STEREO_SOUND */
 		count--;
 	} while (count);
-//#if defined (USE_DOSSOUND)
-//	cur_val += 32 * atari_speaker;
-//#endif
+/*
+#if defined (USE_DOSSOUND)
+	cur_val += 32 * atari_speaker;
+#endif
+*/
 
 	/* loop until the buffer is filled */
 	while (n) {
@@ -786,29 +763,29 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 		if (next_event != SAMPLE) {
 			/* shift the polynomial counters */
 
-		count = Num_pokeys;
-		do {
-			/* decrement all counters by the smallest count found */
-			/* again, no loop for efficiency */
-			div_n_ptr--;
-			*div_n_ptr -= event_min;
-			div_n_ptr--;
-			*div_n_ptr -= event_min;
-			div_n_ptr--;
-			*div_n_ptr -= event_min;
-			div_n_ptr--;
-			*div_n_ptr -= event_min;
+			count = Num_pokeys;
+			do {
+				/* decrement all counters by the smallest count found */
+				/* again, no loop for efficiency */
+				div_n_ptr--;
+				*div_n_ptr -= event_min;
+				div_n_ptr--;
+				*div_n_ptr -= event_min;
+				div_n_ptr--;
+				*div_n_ptr -= event_min;
+				div_n_ptr--;
+				*div_n_ptr -= event_min;
 
-			count--;
-		} while (count);
+				count--;
+			} while (count);
 
 
-                WRITE_U32(samp_cnt_w_ptr,READ_U32(samp_cnt_w_ptr) - event_min);
+			WRITE_U32(samp_cnt_w_ptr, READ_U32(samp_cnt_w_ptr) - event_min);
 
-		/* since the polynomials require a mod (%) function which is
-		   division, I don't adjust the polynomials on the SAMPLE events,
-		   only the CHAN events.  I have to keep track of the change,
-		   though. */
+			/* since the polynomials require a mod (%) function which is
+			   division, I don't adjust the polynomials on the SAMPLE events,
+			   only the CHAN events.  I have to keep track of the change,
+			   though. */
 
 			P4 = (P4 + event_min) % POLY4_SIZE;
 			P5 = (P5 + event_min) % POLY5_SIZE;
@@ -869,10 +846,14 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 						/* if on, turn it off */
 						Outvol[next_event & 0xfd] = 0;
 #ifdef STEREO_SOUND
-						if( (next_event & 0x04) )
+#ifdef __PLUS
+						if (stereo_enabled && (next_event & 0x04))
+#else
+						if ((next_event & 0x04))
+#endif
 							cur_val2 -= AUDV[next_event & 0xfd];
 						else
-#endif
+#endif /* STEREO_SOUND */
 							cur_val -= AUDV[next_event & 0xfd];
 					}
 				}
@@ -887,10 +868,14 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 						/* if on, turn it off */
 						Outvol[next_event & 0xfd] = 0;
 #ifdef STEREO_SOUND
-						if( (next_event & 0x04) )
+#ifdef __PLUS
+						if (stereo_enabled && (next_event & 0x04))
+#else
+						if ((next_event & 0x04))
+#endif
 							cur_val2 -= AUDV[next_event & 0xfd];
 						else
-#endif
+#endif /* STEREO_SOUND */
 							cur_val -= AUDV[next_event & 0xfd];
 					}
 				}
@@ -901,10 +886,14 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 				if (*out_ptr) {
 					/* remove this channel from the signal */
 #ifdef STEREO_SOUND
-					if( (next_event & 0x04) )
+#ifdef __PLUS
+					if (stereo_enabled && (next_event & 0x04))
+#else
+					if ((next_event & 0x04))
+#endif
 						cur_val2 -= AUDV[next_event];
 					else
-#endif
+#endif /* STEREO_SOUND */
 						cur_val -= AUDV[next_event];
 
 					/* and turn the output off */
@@ -916,10 +905,14 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 
 					/* and add it to the output signal */
 #ifdef STEREO_SOUND
-					if( (next_event & 0x04) )
+#ifdef __PLUS
+					if (stereo_enabled && (next_event & 0x04))
+#else
+					if ((next_event & 0x04))
+#endif
 						cur_val2 += AUDV[next_event];
 					else
-#endif
+#endif /* STEREO_SOUND */
 						cur_val += AUDV[next_event];
 				}
 			}
@@ -946,6 +939,9 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 			else
 				iout = cur_val;
 #ifdef STEREO_SOUND
+#ifdef __PLUS
+		if (stereo_enabled)
+#endif
 			if (cur_val2 != last_val2) {
 				if (*Samp_n_cnt < Samp_n_max) {		/* need interpolation */
 					iout2 = (cur_val2 * (*Samp_n_cnt) +
@@ -962,47 +958,58 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 #else   /* INTERPOLATE_SOUND */
 			iout = cur_val;
 #ifdef STEREO_SOUND
+#ifdef __PLUS
+		if (stereo_enabled)
+#endif
 			iout2 = cur_val2;
 #endif  /* STEREO_SOUND */
 #endif  /* INTERPOLATE_SOUND */
 
 #ifdef VOL_ONLY_SOUND
-			if( sampbuf_rptr!=sampbuf_ptr )
-			{ int l;
-				if( sampbuf_cnt[sampbuf_rptr]>0 )
-					sampbuf_cnt[sampbuf_rptr]-=1280;
-				while(  (l=sampbuf_cnt[sampbuf_rptr])<=0 )
-				{	sampout=sampbuf_val[sampbuf_rptr];
-					sampbuf_rptr++;
-					if( sampbuf_rptr>=SAMPBUF_MAX )
-						sampbuf_rptr=0;
-					if( sampbuf_rptr!=sampbuf_ptr )
-					{   
-					    sampbuf_cnt[sampbuf_rptr]+=l;
+#ifdef __PLUS
+			if (g_Sound.nDigitized)
+#endif
+			{
+				if (sampbuf_rptr != sampbuf_ptr) {
+					int l;
+					if (sampbuf_cnt[sampbuf_rptr] > 0)
+						sampbuf_cnt[sampbuf_rptr] -= 1280;
+					while ((l = sampbuf_cnt[sampbuf_rptr]) <= 0) {
+						sampout = sampbuf_val[sampbuf_rptr];
+						sampbuf_rptr++;
+						if (sampbuf_rptr >= SAMPBUF_MAX)
+							sampbuf_rptr = 0;
+						if (sampbuf_rptr != sampbuf_ptr)
+							sampbuf_cnt[sampbuf_rptr] += l;
+						else
+							break;
 					}
-					else	break;
 				}
-			}
-			iout+=sampout;
+				iout += sampout;
 #ifdef STEREO_SOUND
-			if( sampbuf_rptr2!=sampbuf_ptr2 )
-			{ int l;
-				if( sampbuf_cnt2[sampbuf_rptr2]>0 )
-					sampbuf_cnt2[sampbuf_rptr2]-=1280;
-				while(  (l=sampbuf_cnt2[sampbuf_rptr2])<=0 )
-				{	sampout2=sampbuf_val2[sampbuf_rptr2];
-					sampbuf_rptr2++;
-					if( sampbuf_rptr2>=SAMPBUF_MAX )
-						sampbuf_rptr2=0;
-					if( sampbuf_rptr2!=sampbuf_ptr2 )
-					{   
-					    sampbuf_cnt2[sampbuf_rptr2]+=l;
+#ifdef __PLUS
+				if (stereo_enabled)
+#endif
+				{
+					if (sampbuf_rptr2 != sampbuf_ptr2) {
+						int l;
+						if (sampbuf_cnt2[sampbuf_rptr2] > 0)
+							sampbuf_cnt2[sampbuf_rptr2] -= 1280;
+						while ((l = sampbuf_cnt2[sampbuf_rptr2]) <= 0) {
+							sampout2 = sampbuf_val2[sampbuf_rptr2];
+							sampbuf_rptr2++;
+							if (sampbuf_rptr2 >= SAMPBUF_MAX)
+								sampbuf_rptr2 = 0;
+							if (sampbuf_rptr2 != sampbuf_ptr2)
+								sampbuf_cnt2[sampbuf_rptr2] += l;
+							else
+								break;
+						}
 					}
-					else	break;
+					iout2 += sampout2;
 				}
-			}
-			iout2+=sampout2;
 #endif  /* STEREO_SOUND */
+			}
 #endif  /* VOL_ONLY_SOUND */
 
 #ifdef CLIP_SOUND
@@ -1016,20 +1023,38 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 				*buffer++ = (uint8) iout;
 			}
 #ifdef STEREO_SOUND
-			if ((stereo_enabled ? iout2 : iout) > SAMP_MAX) {	/* then check high limit */
-				*buffer++ = (uint8) SAMP_MAX;	/* and limit if greater */
+#ifdef __PLUS
+			if (stereo_enabled) {
+				if (iout2 > SAMP_MAX)
+					*buffer++ = (uint8) SAMP_MAX;
+				else if (iout2 < SAMP_MIN)
+					*buffer++ = (uint8) SAMP_MIN;
+				else
+					*buffer++ = (uint8) iout2;
 			}
-			else if ((stereo_enabled ? iout2 : iout) < SAMP_MIN) {		/* else check low limit */
-				*buffer++ = (uint8) SAMP_MIN;	/* and limit if less */
-			}
-			else {				/* otherwise use raw value */
-				*buffer++ = (uint8) (stereo_enabled ? iout2 : iout);
+#else /* __PLUS */
+			if (Num_pokeys > 1) {
+				if ((stereo_enabled ? iout2 : iout) > SAMP_MAX) {	/* then check high limit */
+					*buffer++ = (uint8) SAMP_MAX;	/* and limit if greater */
+				}
+				else if ((stereo_enabled ? iout2 : iout) < SAMP_MIN) {		/* else check low limit */
+					*buffer++ = (uint8) SAMP_MIN;	/* and limit if less */
+				}
+				else {				/* otherwise use raw value */
+					*buffer++ = (uint8) (stereo_enabled ? iout2 : iout);
+				}
 			}
+#endif /* __PLUS */
 #endif /* STEREO_SOUND */
 #else /* CLIP_SOUND */
 			*buffer++ = (uint8) iout;	/* clipping not selected, use value */
 #ifdef STEREO_SOUND
-			*buffer++ = (uint8) (stereo_enabled ? iout2 : iout);
+			if (Num_pokeys > 1)
+#ifdef ASAP
+				*buffer++ = (uint8) iout2;
+#else
+				*buffer++ = (uint8) (stereo_enabled ? iout2 : iout);
+#endif
 #endif /* STEREO_SOUND */
 #endif /* CLIP_SOUND */
 
@@ -1041,119 +1066,141 @@ void Pokey_process_8(void * sndbuffer, unsigned sndn)
 			/* and indicate one less byte in the buffer */
 			n--;
 #ifdef STEREO_SOUND
-			n--;
+#ifdef __PLUS
+			if (stereo_enabled)
+#endif
+			if (Num_pokeys > 1)
+				n--;
 #endif
 		}
 	}
 #ifdef VOL_ONLY_SOUND
-	if( sampbuf_rptr==sampbuf_ptr )
-		sampbuf_last=cpu_clock;
-#ifdef STEREO_SOUND
-	if( sampbuf_rptr2==sampbuf_ptr2 )
-		sampbuf_last2=cpu_clock;
+#ifdef __PLUS
+	if (g_Sound.nDigitized)
 #endif
+	{
+		if (sampbuf_rptr == sampbuf_ptr)
+			sampbuf_last = cpu_clock;
+#ifdef STEREO_SOUND
+#ifdef __PLUS
+	if (stereo_enabled)
+#endif
+		if (sampbuf_rptr2 == sampbuf_ptr2)
+			sampbuf_last2 = cpu_clock;
+#endif /* STEREO_SOUND */
+	}
 #endif  /* VOL_ONLY_SOUND */
-
-	if( IsSoundFileOpen())
-		WriteToSoundFile(sndbuffer, sndn);
 }
 
 #ifdef SERIO_SOUND
-static void Update_serio_sound_rf( int out, UBYTE data )
+static void Update_serio_sound_rf(int out, UBYTE data)
 {
 #ifdef VOL_ONLY_SOUND
-   int bits,pv,future;
-        if (!serio_sound_enabled) return;
+#ifdef __PLUS
+	if (g_Sound.nDigitized) {
+#endif
+	int bits, pv, future;
+	if (!serio_sound_enabled) return;
 
-	pv=0;
-	future=0;
-	bits= (data<<1) | 0x200;
-	while( bits )
+	pv = 0;
+	future = 0;
+	bits = (data << 1) | 0x200;
+	while (bits)
 	{
-		sampbuf_lastval-=pv;
-		pv=(bits&0x01)*AUDV[3];	/* FIXME!!! - set volume from AUDV */
-		sampbuf_lastval+=pv;
+		sampbuf_lastval -= pv;
+		pv = (bits & 0x01) * AUDV[3];	/* FIXME!!! - set volume from AUDV */
+		sampbuf_lastval += pv;
 
-	sampbuf_val[sampbuf_ptr]=sampbuf_lastval;
-	sampbuf_cnt[sampbuf_ptr]=
-		(cpu_clock+future-sampbuf_last)*128*samp_freq/178979;
-	sampbuf_last=cpu_clock+future;
+	sampbuf_val[sampbuf_ptr] = sampbuf_lastval;
+	sampbuf_cnt[sampbuf_ptr] =
+		(cpu_clock + future-sampbuf_last) * 128 * samp_freq / 178979;
+	sampbuf_last = cpu_clock + future;
 	sampbuf_ptr++;
-	if( sampbuf_ptr>=SAMPBUF_MAX )
-		sampbuf_ptr=0;
-	if( sampbuf_ptr==sampbuf_rptr )
-	{	sampbuf_rptr++;
-		if( sampbuf_rptr>=SAMPBUF_MAX )
-			sampbuf_rptr=0;
+	if (sampbuf_ptr >= SAMPBUF_MAX )
+		sampbuf_ptr = 0;
+	if (sampbuf_ptr == sampbuf_rptr ) {
+		sampbuf_rptr++;
+		if (sampbuf_rptr >= SAMPBUF_MAX)
+			sampbuf_rptr = 0;
 	}
-			/* 1789790/19200 = 93 */
-		future+=93;	/* ~ 19200 bit/s - FIXME!!! set speed form AUDF [2] ??? */
-		bits>>=1;
+		/* 1789790/19200 = 93 */
+		future += 93;	/* ~ 19200 bit/s - FIXME!!! set speed form AUDF [2] ??? */
+		bits >>= 1;
 	}
-	sampbuf_lastval-=pv;
+	sampbuf_lastval -= pv;
+#ifdef __PLUS
+	}
+#endif
 #endif  /* VOL_ONLY_SOUND */
 }
 #endif /* SERIO_SOUND */
 
-void Pokey_process_16(void * sndbuffer, unsigned sndn)
+static void Pokey_process_16(void *sndbuffer, unsigned sndn)
 {
-  uint16 *buffer = sndbuffer;
-  int i;
+	uint16 *buffer = (uint16 *) sndbuffer;
+	int i;
 
-  Pokey_process_8(buffer, sndn);
+	Pokey_process_8(buffer, sndn);
 
-  for (i = sndn - 1; i >= 0; i--)
-  {
-    int smp = ((int)(((uint8 *)buffer)[i]) - 0x80) * 0x100;
+	for (i = sndn - 1; i >= 0; i--) {
+		int smp = ((int) (((uint8 *) buffer)[i]) - 0x80) * 0x100;
 
-    if (smp > SHRT_MAX)
-      smp = SHRT_MAX;
-    if (smp < SHRT_MIN)
-      smp = SHRT_MIN;
+		if (smp > 32767)
+			smp = 32767;
+		else if (smp < -32768)
+			smp = -32768;
 
-    buffer[i] = smp;
-  }
+		buffer[i] = smp;
+	}
 }
 
 #ifdef CONSOLE_SOUND
 static void Update_consol_sound_rf(int set)
-{ 
+{
 #ifdef VOL_ONLY_SOUND
-  static int prev_atari_speaker=0;
-  static unsigned int prev_cpu_clock=0;
-  int d;
-        if (!console_sound_enabled) return;
-        
-	if( !set && samp_consol_val==0 )	return;
-	sampbuf_lastval-=samp_consol_val;
-	if( prev_atari_speaker!=atari_speaker )
-	{	samp_consol_val=atari_speaker*8*4;	/* gain */
-		prev_cpu_clock=cpu_clock;
-	}
-	else if( !set )
-	{	d=cpu_clock - prev_cpu_clock;
-		if( d<114 )
-		{	sampbuf_lastval+=samp_consol_val;   return;	}
-		while( d>=114 /* CPUL */ )
-		{	samp_consol_val=samp_consol_val*99/100;
-			d-=114;
-		}
-		prev_cpu_clock=cpu_clock-d;
-	}
-	sampbuf_lastval+=samp_consol_val;
-	prev_atari_speaker=atari_speaker;
+	static int prev_atari_speaker = 0;
+	static unsigned int prev_cpu_clock = 0;
+	int d;
+#ifdef __PLUS
+	if (!g_Sound.nDigitized)
+		return;
+#endif
+	if (!console_sound_enabled)
+		return;
 
-	sampbuf_val[sampbuf_ptr]=sampbuf_lastval;
-	sampbuf_cnt[sampbuf_ptr]=
-		(cpu_clock-sampbuf_last)*128*samp_freq/178979;
-	sampbuf_last=cpu_clock;
+	if (!set && samp_consol_val == 0)
+		return;
+	sampbuf_lastval -= samp_consol_val;
+	if (prev_atari_speaker != atari_speaker) {
+		samp_consol_val = atari_speaker * 8 * 4;	/* gain */
+		prev_cpu_clock = cpu_clock;
+	}
+	else if (!set) {
+		d = cpu_clock - prev_cpu_clock;
+		if (d < 114) {
+			sampbuf_lastval += samp_consol_val;
+			return;
+		}
+		while (d >= 114 /* CPUL */) {
+			samp_consol_val = samp_consol_val * 99 / 100;
+			d -= 114;
+		}
+		prev_cpu_clock = cpu_clock - d;
+	}
+	sampbuf_lastval += samp_consol_val;
+	prev_atari_speaker = atari_speaker;
+
+	sampbuf_val[sampbuf_ptr] = sampbuf_lastval;
+	sampbuf_cnt[sampbuf_ptr] =
+		(cpu_clock - sampbuf_last) * 128 * samp_freq / 178979;
+	sampbuf_last = cpu_clock;
 	sampbuf_ptr++;
-	if( sampbuf_ptr>=SAMPBUF_MAX )
-		sampbuf_ptr=0;
-	if( sampbuf_ptr==sampbuf_rptr )
-	{	sampbuf_rptr++;
-		if( sampbuf_rptr>=SAMPBUF_MAX )
-			sampbuf_rptr=0;
+	if (sampbuf_ptr >= SAMPBUF_MAX)
+		sampbuf_ptr = 0;
+	if (sampbuf_ptr == sampbuf_rptr) {
+		sampbuf_rptr++;
+		if (sampbuf_rptr >= SAMPBUF_MAX)
+			sampbuf_rptr = 0;
 	}
 #endif  /* VOL_ONLY_SOUND */
 }
@@ -1167,54 +1214,3 @@ static void Update_vol_only_sound_rf(void)
 #endif /* CONSOLE_SOUND */
 }
 #endif  /* VOL_ONLY_SOUND */
-
-/*
-$Log: pokeysnd.c,v $
-Revision 1.18  2003/12/12 00:23:59  markgrebe
-Added enable for console and sio sound
-
-Revision 1.17  2003/02/24 09:33:09  joy
-header cleanup
-
-Revision 1.16  2003/02/19 14:07:47  joy
-configure stuff cleanup
-
-Revision 1.15  2003/02/10 14:59:17  joy
-fixes fixes
-
-Revision 1.14  2003/02/10 13:29:16  joy
-extern removed
-
-Revision 1.13  2003/02/09 21:20:43  joy
-updated for global enable_new_pokey
-
-Revision 1.12  2003/02/09 13:17:29  joy
-switch Pokey cores on-the-fly
-
-Revision 1.11  2003/02/08 23:52:17  joy
-little cleanup
-
-Revision 1.10  2002/12/08 20:32:02  knik
-added 16bit and quality sound setting support
-added multiple sound engines support
-
-Revision 1.9  2002/08/26 05:44:22  pfusik
-Adam Bienias's fix for better sound quality
-
-Revision 1.8  2001/07/22 08:24:47  knik
-PURE -> PURETONE to avoid windows headers interference
-
-Revision 1.7  2001/07/19 23:25:05  fox
-using poly9_lookup and poly17_lookup (initialised by pokey.c), removed bit17
-(which was initialised with rand()) saving ca. 100 KB of memory.
-
-Revision 1.5  2001/04/15 09:17:53  knik
-atari800_big_endian -> words_bigendian (autoconf compatibility)
-
-Revision 1.4  2001/04/08 06:05:24  knik
-weird bug in WRITE_U32 macro fixed
-
-Revision 1.3  2001/03/18 06:34:58  knik
-WIN32 conditionals removed
-
-*/
