@@ -2,7 +2,7 @@
  * keyboard.c - Win32 port specific code
  *
  * Copyright (C) 2000 Krzysztof Nikiel
- * Copyright (C) 2000-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2000-2005 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -22,11 +22,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "config.h"
 #define DIRECTINPUT_VERSION	    0x0500
-
 #include <windows.h>
 #include <dinput.h>
+
 #include "atari.h"
+
 #include "main.h"
 #include "keyboard.h"
 
@@ -52,7 +54,7 @@ int prockb(void)
 {
   DIDEVICEOBJECTDATA que[KEYBUFSIZE];
   DWORD dwEvents;
-  int i;
+  DWORD i;
   HRESULT hRes;
 
   dwEvents = KEYBUFSIZE;
@@ -177,13 +179,3 @@ void clearkb(void)
   pause_hit = 0;
   kbcode = 0;
 }
-
-/*
-$Log: keyboard.c,v $
-Revision 1.2  2003/02/24 09:33:32  joy
-header cleanup
-
-Revision 1.1  2001/03/18 07:56:48  knik
-win32 port
-
-*/
