@@ -2,7 +2,7 @@
  * atari_wince.c - WinCE port specific code
  *
  * Copyright (C) 2001 Vasyl Tsvirkunov
- * Copyright (C) 2001-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2001-2006 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -30,10 +30,11 @@ extern "C"
 #include "input.h"
 #include "main.h"
 #include "keyboard.h"
-#include "screen.h"
+#include "screen_wince.h"
 #include "ui.h"
 
 int virtual_joystick = 0;
+int smkeyhack = 0;
 };
 
 int currentKeyboardMode = 4;
@@ -289,6 +290,7 @@ sKeyTranslation kbd_translation[] =
 	{0, AKEY_WARMSTART},
 	{0, AKEY_COLDSTART},
 	{0, AKEY_BREAK},
+	{VK_RETURN, AKEY_RETURN},
 // Non-mappable entries
 	{VK_SHIFT, AKEY_SHFT},
 	{VK_CONTROL, AKEY_CTRL},
@@ -334,7 +336,6 @@ sKeyTranslation kbd_translation[] =
 	{VK_BACKQUOTE, AKEY_ATARI},
 	{VK_CAPITAL, AKEY_CAPSTOGGLE},
 	{VK_TAB, AKEY_TAB},
-	{VK_RETURN, AKEY_RETURN},
 	{VK_SPACE, AKEY_SPACE},
 	{VK_LBRACKET, AKEY_LESS},
 	{VK_RBRACKET, AKEY_GREATER},
@@ -366,6 +367,7 @@ sKeyTranslation kbd_translation[] =
 #define KBDT_WARMSTART	11
 #define KBDT_COLDSTART	12
 #define KBDT_BREAK		13
+#define KBDT_RETURN		14
 
 
 void reset_kbd()
@@ -435,7 +437,7 @@ void hitbutton(short code)
 		else if(code == joykey_map[get_screen_mode()][3])
 			kbcode = AKEY_RIGHT;
 		else if(code == klist.vkStart)
-			kbcode = AKEY_TAB;
+			kbcode = AKEY_BACKSPACE;
 		else if(code == klist.vkA)
 			kbcode = AKEY_SPACE;
 		else if(code == klist.vkB)
@@ -456,10 +458,14 @@ void hitbutton(short code)
 			stick0 &= ~4;
 		else if(code == joykey_map[get_screen_mode()][3])
 			stick0 &= ~8;
-		else if(code == klist.vkA || code == klist.vkB)
+		else if(code == klist.vkA || code == klist.vkB || ((code == '4' || code == '6') && issmartphone))
 			trig0 = 0;
+		else if(code == '5' && issmartphone)
+			key_shift = 1;
 		else if(code == klist.vkC)
 			kbcode = AKEY_UI;
+		else if ((code == VK_F3) && (issmartphone))
+			set_screen_mode(get_screen_mode()+1); 
 		else
 		for(int i=0; i<sizeof(kbd_translation)/sizeof(kbd_translation[0]); i++)
 			if(code == kbd_translation[i].winKey)
@@ -494,7 +500,7 @@ void releasebutton(short code)
 		else if(code == klist.vkRight)
 			kbcode = AKEY_RIGHT;
 		else if(code == klist.vkStart)
-			kbcode = AKEY_TAB;
+			kbcode = AKEY_BACKSPACE;
 		else if(code == klist.vkA)
 			kbcode = AKEY_SPACE;
 		else if(code == klist.vkB)
@@ -514,8 +520,10 @@ void releasebutton(short code)
 			stick0 |= 4;
 		else if(code == joykey_map[get_screen_mode()][3])
 			stick0 |= 8;
-		else if(code == klist.vkA || code == klist.vkB)
+		else if(code == klist.vkA || code == klist.vkB || ((code == '4' || code == '6') && issmartphone))
 			trig0 = 1;
+		else if(code == '5' && issmartphone)
+			key_shift = 0;
 		else if(code == klist.vkC)
 			kbcode = AKEY_UI;
 		else
@@ -632,9 +640,6 @@ void tapscreen(short x, short y)
 		}
 	}
 
-	if(kbcode == AKEY_BREAK)
-		key_break = 1;
-
 	push_key(kbcode);
 }
 
@@ -751,9 +756,6 @@ void untapscreen(short x, short y)
 		}
 	}
 
-	if(kbcode == AKEY_BREAK)
-		key_break = 0;
-	
 	release_key(kbcode);
 }
 
@@ -783,7 +785,6 @@ void dragscreen(short x, short y)
 	}
 }
 
-
 void push_key(short akey)
 {
 	switch(akey)
@@ -880,9 +881,29 @@ int initinput(void)
 	joykey_map[2][2] = klist.vkUp;
 	joykey_map[2][3] = klist.vkDown;
 
-	kbd_translation[KBDT_F3].winKey = klist.vkA;
-	kbd_translation[KBDT_F2].winKey = klist.vkB;
-	kbd_translation[KBDT_UI].winKey = klist.vkC;
+	/* TODO: REMOVE THIS ASAP */
+	if (smkeyhack)
+	{
+		klist.vkB ^= klist.vkC;
+		klist.vkC ^= klist.vkB;
+		klist.vkB ^= klist.vkC;
+	}
+
+	if (issmartphone)
+	{
+		kbd_translation[KBDT_F3].winKey = '8';
+		kbd_translation[KBDT_F2].winKey = '7';
+		kbd_translation[KBDT_UI].winKey = klist.vkC;
+		kbd_translation[KBDT_F4].winKey = '9';
+		kbd_translation[KBDT_RETURN].winKey = '0';
+ 
+	}
+	else
+	{
+		kbd_translation[KBDT_F3].winKey = klist.vkA;
+		kbd_translation[KBDT_F2].winKey = klist.vkB;
+		kbd_translation[KBDT_UI].winKey = klist.vkC;
+	}
 
 	kbd_image = kbd_image_800;
 	kbd_struct = kbd_struct_800;
