@@ -2,7 +2,7 @@
  * main.c - WinCE port specific code
  *
  * Copyright (C) 2001 Vasyl Tsvirkunov
- * Copyright (C) 2001-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2001-2006 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -27,10 +27,9 @@
 #include <windows.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <time.h>
 #include "config.h"
 #include "main.h"
-#include "screen.h"
+#include "screen_wince.h"
 #include "keyboard.h"
 #include "sound.h"
 #include "ui.h"
@@ -39,11 +38,14 @@ LPTSTR myname = TEXT("Pocket Atari");
 char* mynameb = "Pocket Atari";
 HWND hWndMain;
 HINSTANCE myInstance;
-
-extern tUIDriver wince_ui_driver;
+char issmartphone = 0;
 
 extern int wince_main(int argc, char **argv);
 
+extern void entire_screen_dirty(void);
+extern UBYTE *screen_dirty;
+//extern void OrientationChanged(void);
+
 static char **gargv = NULL;
 static int gargc = 0;
 
@@ -54,7 +56,6 @@ void __cdecl atari_exit(int code)
 #else
 	MsgPump();
 #endif
-	exit(code);
 }
 
 void wce_perror(char* s)
@@ -68,8 +69,7 @@ void wce_perror(char* s)
 
 DWORD WINAPI vloop(LPVOID p)
 {
-	srand(time(0));
-	ui_driver = &wince_ui_driver;
+	srand(GetTickCount());
 	wince_main(gargc, gargv);
 	atari_exit(0);
 	return 0;
@@ -115,15 +115,18 @@ static long FAR PASCAL WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM
 	case WM_DESTROY:
 		PostQuitMessage(0);
 		return 0;
-		
 	case WM_SETFOCUS:
 	case WM_ACTIVATE:
+		if (screen_dirty) entire_screen_dirty();
 		gr_resume();
 		return 0;
 	case WM_KILLFOCUS:
 	case WM_HIBERNATE:
 		gr_suspend();
 		return 0;
+	case WM_SETTINGCHANGE:
+		OrientationChanged();
+		break;
 	}
 /* In the first version all other events were merrily dropped on floor in singlethreaded
    model. That caused some troubles with debugging and suspected to cause spurious key up
@@ -137,9 +140,7 @@ void MsgPump()
 {
 	MSG msg;
 	while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
-	{
 		WindowProc(msg.hwnd, msg.message, msg.wParam, msg.lParam);
-	}
 }
 #endif
 
@@ -147,19 +148,19 @@ void MsgPump()
 static BOOL initwin(HINSTANCE hInstance, int nCmdShow)
 {
 	WNDCLASS wc;
-	
+
 	wc.style = CS_HREDRAW | CS_VREDRAW;
 	wc.lpfnWndProc = WindowProc;
 	wc.cbClsExtra = 0;
 	wc.cbWndExtra = 0;
 	wc.hInstance = hInstance;
-	wc.hIcon = NULL; //LoadIcon(hInstance, IDI_APPLICATION);
+	wc.hIcon = NULL;
 	wc.hCursor = NULL;
 	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
 	wc.lpszMenuName = NULL;
 	wc.lpszClassName = myname;
 	RegisterClass(&wc);
-	
+
 	hWndMain = CreateWindow(myname,
 		myname,
 		WS_VISIBLE,
@@ -171,11 +172,12 @@ static BOOL initwin(HINSTANCE hInstance, int nCmdShow)
 		NULL,
 		hInstance,
 		NULL);
-	
+
 	if(!hWndMain)
 		return 1;
-	
-	SetWindowPos(hWndMain, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
+
+	SetWindowPos(hWndMain, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
+	UpdateWindow(hWndMain);
 	return 0;
 }
 
@@ -185,18 +187,30 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLin
 	MSG msg;
 	HANDLE hth;
 #endif
-	
+
 	int i;
 	static int argc = 0;
 	static char args[0x400];
 	static char *argv[100];
+	TCHAR platform[100];
 
 	myInstance = hInstance;
 	if(initwin(hInstance, nCmdShow))
 	{
 		return 1;
 	}
-	
+#ifndef MULTITHREADED
+	MsgPump();
+#endif
+
+	if (SystemParametersInfo(SPI_GETPLATFORMTYPE, 100, platform, 0))
+	{
+		if (wcsstr(platform, _T("mart")))
+			issmartphone = 1;
+	}
+	else
+		issmartphone = 1; /* it probably is a smartphone */
+
 	if(lpCmdLine)
 		WideCharToMultiByte(CP_ACP, 0, lpCmdLine, -1, args, 0x400, NULL, NULL);
 	else
@@ -217,20 +231,20 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLin
 		args[i] = 0;
 	}
 	argv[argc] = NULL;
-	
+
 	gargv = argv;
 	gargc = argc;
-	
+
 #ifdef MULTITHREADED
 	if((hth = CreateThread(NULL, 0x10000, vloop, NULL, 0, NULL)) == NULL)
 		return 1;
-	
+
 	while(GetMessage(&msg, NULL, 0, 0))
 	{
 		TranslateMessage(&msg);
 		DispatchMessage(&msg);
 	}
-	
+
 	/* wait for the other thread to exit */
 	WaitForSingleObject((HANDLE)hth, INFINITE);
 	return msg.wParam;
